@@ -17,14 +17,12 @@ limitations under the License.
 package ceph
 
 import (
+	"flag"
+	"os"
+
 	"github.com/pkg/errors"
 	"github.com/rook/rook/cmd/rook/rook"
-	"github.com/rook/rook/pkg/clusterd"
-	"github.com/rook/rook/pkg/daemon/ceph/agent/flexvolume/attachment"
 	operator "github.com/rook/rook/pkg/operator/ceph"
-	cluster "github.com/rook/rook/pkg/operator/ceph/cluster"
-	"github.com/rook/rook/pkg/operator/ceph/csi"
-
 	opcontroller "github.com/rook/rook/pkg/operator/ceph/controller"
 	"github.com/rook/rook/pkg/operator/k8sutil"
 	"github.com/rook/rook/pkg/util/flags"
@@ -43,38 +41,31 @@ https://github.com/rook/rook`,
 }
 
 func init() {
-	// csi deployment templates
-	operatorCmd.Flags().StringVar(&csi.RBDPluginTemplatePath, "csi-rbd-plugin-template-path", csi.DefaultRBDPluginTemplatePath, "path to ceph-csi rbd plugin template")
-
-	operatorCmd.Flags().StringVar(&csi.RBDProvisionerDepTemplatePath, "csi-rbd-provisioner-dep-template-path", csi.DefaultRBDProvisionerDepTemplatePath, "path to ceph-csi rbd provisioner deployment template")
-
-	operatorCmd.Flags().StringVar(&csi.CephFSPluginTemplatePath, "csi-cephfs-plugin-template-path", csi.DefaultCephFSPluginTemplatePath, "path to ceph-csi cephfs plugin template")
-	operatorCmd.Flags().StringVar(&csi.CephFSProvisionerDepTemplatePath, "csi-cephfs-provisioner-dep-template-path", csi.DefaultCephFSProvisionerDepTemplatePath, "path to ceph-csi cephfs provisioner deployment template")
-
-	operatorCmd.Flags().BoolVar(&cluster.EnableMachineDisruptionBudget, "enable-machine-disruption-budget", false, "enable fencing controllers")
+	operatorCmd.Flags().BoolVar(&operator.EnableMachineDisruptionBudget, "enable-machine-disruption-budget", false, "enable fencing controllers")
 
 	flags.SetFlagsFromEnv(operatorCmd.Flags(), rook.RookEnvVarPrefix)
-	flags.SetLoggingFlags(operatorCmd.Flags())
+	operatorCmd.Flags().AddGoFlagSet(flag.CommandLine)
+	if err := operatorCmd.Flags().Parse(nil); err != nil {
+		panic(err)
+	}
 	operatorCmd.RunE = startOperator
 }
 
 func startOperator(cmd *cobra.Command, args []string) error {
-
 	rook.SetLogLevel()
-
 	rook.LogStartupInfo(operatorCmd.Flags())
 
 	logger.Info("starting Rook-Ceph operator")
 	context := createContext()
-	context.NetworkInfo = clusterd.NetworkInfo{}
 	context.ConfigDir = k8sutil.DataDir
-	volumeAttachment, err := attachment.New(context)
-	if err != nil {
-		rook.TerminateFatal(err)
+
+	// Fail if operator namespace is not provided
+	if os.Getenv(k8sutil.PodNamespaceEnvVar) == "" {
+		rook.TerminateFatal(errors.Errorf("rook operator namespace is not provided. expose it via downward API in the rook operator manifest file using environment variable %q", k8sutil.PodNamespaceEnvVar))
 	}
 
-	rook.CheckOperatorResources(context.Clientset)
-	rookImage := rook.GetOperatorImage(context.Clientset, containerName)
+	rook.CheckOperatorResources(cmd.Context(), context.Clientset)
+	rookImage := rook.GetOperatorImage(cmd.Context(), context.Clientset, containerName)
 	rookBaseImageCephVersion, err := rook.GetOperatorBaseImageCephVersion(context)
 	if err != nil {
 		logger.Errorf("failed to get operator base image ceph version. %v", err)
@@ -82,11 +73,11 @@ func startOperator(cmd *cobra.Command, args []string) error {
 	opcontroller.OperatorCephBaseImageVersion = rookBaseImageCephVersion
 	logger.Infof("base ceph version inside the rook operator image is %q", opcontroller.OperatorCephBaseImageVersion)
 
-	serviceAccountName := rook.GetOperatorServiceAccount(context.Clientset)
-	op := operator.New(context, volumeAttachment, rookImage, serviceAccountName)
+	serviceAccountName := rook.GetOperatorServiceAccount(cmd.Context(), context.Clientset)
+	op := operator.New(context, rookImage, serviceAccountName)
 	err = op.Run()
 	if err != nil {
-		rook.TerminateFatal(errors.Wrap(err, "failed to run operator\n"))
+		rook.TerminateFatal(errors.Wrap(err, "failed to run operator"))
 	}
 
 	return nil
