@@ -34,7 +34,7 @@ const (
 	confirmFlag             = "--yes-i-really-mean-it"
 	reallyConfirmFlag       = "--yes-i-really-really-mean-it"
 	targetSizeRatioProperty = "target_size_ratio"
-	compressionModeProperty = "compression_mode"
+	CompressionModeProperty = "compression_mode"
 	PgAutoscaleModeProperty = "pg_autoscale_mode"
 	PgAutoscaleModeOn       = "on"
 )
@@ -49,12 +49,12 @@ type CephStoragePoolDetails struct {
 	Number                 int     `json:"pool_id"`
 	Size                   uint    `json:"size"`
 	ErasureCodeProfile     string  `json:"erasure_code_profile"`
-	FailureDomain          string  `json:"failureDomain"`
 	CrushRoot              string  `json:"crushRoot"`
 	DeviceClass            string  `json:"deviceClass"`
 	CompressionMode        string  `json:"compression_mode"`
 	TargetSizeRatio        float64 `json:"target_size_ratio,omitempty"`
 	RequireSafeReplicaSize bool    `json:"requireSafeReplicaSize,omitempty"`
+	CrushRule              string  `json:"crush_rule"`
 }
 
 type CephStoragePoolStats struct {
@@ -116,6 +116,30 @@ func GetPoolNamesByID(context *clusterd.Context, clusterInfo *ClusterInfo) (map[
 	return names, nil
 }
 
+func getPoolApplication(context *clusterd.Context, clusterInfo *ClusterInfo, poolName string) (string, error) {
+	args := []string{"osd", "pool", "application", "get", poolName}
+	appDetails, err := NewCephCommand(context, clusterInfo, args).Run()
+	if err != nil {
+		return "", errors.Wrapf(err, "failed to get current application for pool %s", poolName)
+	}
+
+	if len(appDetails) == 0 {
+		// no application name
+		return "", nil
+	}
+	var application map[string]interface{}
+	err = json.Unmarshal([]byte(appDetails), &application)
+	if err != nil {
+		return "", errors.Wrapf(err, "unmarshal failed raw buffer response %s", string(appDetails))
+	}
+	for name := range application {
+		// Return the first application name in the list since only one is expected
+		return name, nil
+	}
+	// No application name assigned
+	return "", nil
+}
+
 // GetPoolDetails gets all the details of a given pool
 func GetPoolDetails(context *clusterd.Context, clusterInfo *ClusterInfo, name string) (CephStoragePoolDetails, error) {
 	args := []string{"osd", "pool", "get", name, "all"}
@@ -128,7 +152,6 @@ func GetPoolDetails(context *clusterd.Context, clusterInfo *ClusterInfo, name st
 }
 
 func ParsePoolDetails(in []byte) (CephStoragePoolDetails, error) {
-
 	// The response for osd pool get when passing var=all is actually malformed JSON similar to:
 	// {"pool":"rbd","size":1}{"pool":"rbd","min_size":2}...
 	// Note the multiple top level entities, one for each property returned.  To workaround this,
@@ -155,27 +178,29 @@ func ParsePoolDetails(in []byte) (CephStoragePoolDetails, error) {
 	return poolDetails, nil
 }
 
-func CreatePoolWithProfile(context *clusterd.Context, clusterInfo *ClusterInfo, clusterSpec *cephv1.ClusterSpec, poolName string, pool cephv1.PoolSpec, appName string) error {
+func CreatePoolWithProfile(context *clusterd.Context, clusterInfo *ClusterInfo, clusterSpec *cephv1.ClusterSpec, pool cephv1.NamedPoolSpec, appName string) error {
+	if pool.Name == "" {
+		return errors.New("pool name must be specified")
+	}
 	if pool.IsReplicated() {
-		return CreateReplicatedPoolForApp(context, clusterInfo, clusterSpec, poolName, pool, DefaultPGCount, appName)
+		return CreateReplicatedPoolForApp(context, clusterInfo, clusterSpec, pool, DefaultPGCount, appName)
 	}
 
 	if !pool.IsErasureCoded() {
 		// neither a replicated or EC pool
-		return fmt.Errorf("pool %q type is not defined as replicated or erasure coded", poolName)
+		return errors.Errorf("pool %q type is not defined as replicated or erasure coded", pool.Name)
 	}
 
 	// create a new erasure code profile for the new pool
-	ecProfileName := GetErasureCodeProfileForPool(poolName)
-	if err := CreateErasureCodeProfile(context, clusterInfo, ecProfileName, pool); err != nil {
-		return errors.Wrapf(err, "failed to create erasure code profile for pool %q", poolName)
+	ecProfileName := GetErasureCodeProfileForPool(pool.Name)
+	if err := CreateErasureCodeProfile(context, clusterInfo, ecProfileName, pool.PoolSpec); err != nil {
+		return errors.Wrapf(err, "failed to create erasure code profile for pool %q", pool.Name)
 	}
 
 	// If the pool is not a replicated pool, then the only other option is an erasure coded pool.
 	return CreateECPoolForApp(
 		context,
 		clusterInfo,
-		poolName,
 		ecProfileName,
 		pool,
 		DefaultPGCount,
@@ -233,16 +258,25 @@ func DeletePool(context *clusterd.Context, clusterInfo *ClusterInfo, name string
 }
 
 func givePoolAppTag(context *clusterd.Context, clusterInfo *ClusterInfo, poolName, appName string) error {
-	args := []string{"osd", "pool", "application", "enable", poolName, appName, confirmFlag}
-	_, err := NewCephCommand(context, clusterInfo, args).Run()
+	currentAppName, err := getPoolApplication(context, clusterInfo, poolName)
 	if err != nil {
-		return errors.Wrapf(err, "failed to enable application %s on pool %s", appName, poolName)
+		return errors.Wrapf(err, "failed to get application for pool %q", poolName)
+	}
+	if currentAppName == appName {
+		logger.Infof("application %q is already set on pool %q", appName, poolName)
+		return nil
+	}
+
+	args := []string{"osd", "pool", "application", "enable", poolName, appName, confirmFlag}
+	_, err = NewCephCommand(context, clusterInfo, args).Run()
+	if err != nil {
+		return errors.Wrapf(err, "failed to enable application %q on pool %q", appName, poolName)
 	}
 
 	return nil
 }
 
-func setCommonPoolProperties(context *clusterd.Context, clusterInfo *ClusterInfo, pool cephv1.PoolSpec, poolName, appName string) error {
+func setCommonPoolProperties(context *clusterd.Context, clusterInfo *ClusterInfo, pool cephv1.NamedPoolSpec, appName string) error {
 	if len(pool.Parameters) == 0 {
 		pool.Parameters = make(map[string]string)
 	}
@@ -252,63 +286,63 @@ func setCommonPoolProperties(context *clusterd.Context, clusterInfo *ClusterInfo
 	}
 
 	if pool.IsCompressionEnabled() {
-		pool.Parameters[compressionModeProperty] = pool.CompressionMode
+		pool.Parameters[CompressionModeProperty] = pool.CompressionMode
 	}
 
 	// Apply properties
 	for propName, propValue := range pool.Parameters {
-		err := SetPoolProperty(context, clusterInfo, poolName, propName, propValue)
+		err := SetPoolProperty(context, clusterInfo, pool.Name, propName, propValue)
 		if err != nil {
-			logger.Errorf("failed to set property %q to pool %q to %q. %v", propName, poolName, propValue, err)
+			logger.Errorf("failed to set property %q to pool %q to %q. %v", propName, pool.Name, propValue, err)
 		}
 	}
 
 	// ensure that the newly created pool gets an application tag
 	if appName != "" {
-		err := givePoolAppTag(context, clusterInfo, poolName, appName)
+		err := givePoolAppTag(context, clusterInfo, pool.Name, appName)
 		if err != nil {
-			return errors.Wrapf(err, "failed to tag pool %q for application %q", poolName, appName)
+			return errors.Wrapf(err, "failed to tag pool %q for application %q", pool.Name, appName)
 		}
 	}
 
 	// If the pool is mirrored, let's enable mirroring
 	// we don't need to check if the pool is erasure coded or not, mirroring will still work, it will simply be slow
 	if pool.Mirroring.Enabled {
-		err := enablePoolMirroring(context, clusterInfo, pool, poolName)
+		err := enablePoolMirroring(context, clusterInfo, pool)
 		if err != nil {
-			return errors.Wrapf(err, "failed to enable mirroring for pool %q", poolName)
+			return errors.Wrapf(err, "failed to enable mirroring for pool %q", pool.Name)
 		}
 
 		// Schedule snapshots
 		if pool.Mirroring.SnapshotSchedulesEnabled() && clusterInfo.CephVersion.IsAtLeastOctopus() {
-			err = enableSnapshotSchedules(context, clusterInfo, pool, poolName)
+			err = enableSnapshotSchedules(context, clusterInfo, pool)
 			if err != nil {
-				return errors.Wrapf(err, "failed to enable snapshot scheduling for pool %q", poolName)
+				return errors.Wrapf(err, "failed to enable snapshot scheduling for pool %q", pool.Name)
 			}
 		}
 	} else {
 		if pool.Mirroring.Mode == "pool" {
 			// Remove storage cluster peers
-			mirrorInfo, err := GetPoolMirroringInfo(context, clusterInfo, poolName)
+			mirrorInfo, err := GetPoolMirroringInfo(context, clusterInfo, pool.Name)
 			if err != nil {
-				return errors.Wrapf(err, "failed to get mirroring info for the pool %q", poolName)
+				return errors.Wrapf(err, "failed to get mirroring info for the pool %q", pool.Name)
 			}
 			for _, peer := range mirrorInfo.Peers {
 				if peer.UUID != "" {
-					err := removeClusterPeer(context, clusterInfo, poolName, peer.UUID)
+					err := removeClusterPeer(context, clusterInfo, pool.Name, peer.UUID)
 					if err != nil {
-						return errors.Wrapf(err, "failed to remove cluster peer with UUID %q for the pool %q", peer.UUID, poolName)
+						return errors.Wrapf(err, "failed to remove cluster peer with UUID %q for the pool %q", peer.UUID, pool.Name)
 					}
 				}
 			}
 
 			// Disable mirroring
-			err = disablePoolMirroring(context, clusterInfo, poolName)
+			err = disablePoolMirroring(context, clusterInfo, pool.Name)
 			if err != nil {
-				return errors.Wrapf(err, "failed to disable mirroring for pool %q", poolName)
+				return errors.Wrapf(err, "failed to disable mirroring for pool %q", pool.Name)
 			}
 		} else if pool.Mirroring.Mode == "image" {
-			logger.Warningf("manually disable mirroring on images in the pool %q", poolName)
+			logger.Warningf("manually disable mirroring on images in the pool %q", pool.Name)
 		}
 	}
 
@@ -318,28 +352,28 @@ func setCommonPoolProperties(context *clusterd.Context, clusterInfo *ClusterInfo
 		maxBytesQuota, err := resource.ParseQuantity(*pool.Quotas.MaxSize)
 		if err != nil {
 			if err == resource.ErrFormatWrong {
-				return errors.Wrapf(err, "maxSize quota incorrectly formatted for pool %q, valid units include k, M, G, T, P, E, Ki, Mi, Gi, Ti, Pi, Ei", poolName)
+				return errors.Wrapf(err, "maxSize quota incorrectly formatted for pool %q, valid units include k, M, G, T, P, E, Ki, Mi, Gi, Ti, Pi, Ei", pool.Name)
 			}
-			return errors.Wrapf(err, "failed setting quota for pool %q, maxSize quota parse error", poolName)
+			return errors.Wrapf(err, "failed setting quota for pool %q, maxSize quota parse error", pool.Name)
 		}
 		// set max_bytes quota, 0 value disables quota
-		err = setPoolQuota(context, clusterInfo, poolName, "max_bytes", strconv.FormatInt(maxBytesQuota.Value(), 10))
+		err = setPoolQuota(context, clusterInfo, pool.Name, "max_bytes", strconv.FormatInt(maxBytesQuota.Value(), 10))
 		if err != nil {
-			return errors.Wrapf(err, "failed to set max_bytes quota for pool %q", poolName)
+			return errors.Wrapf(err, "failed to set max_bytes quota for pool %q", pool.Name)
 		}
 	} else if pool.Quotas.MaxBytes != nil {
 		// set max_bytes quota, 0 value disables quota
-		err := setPoolQuota(context, clusterInfo, poolName, "max_bytes", strconv.FormatUint(*pool.Quotas.MaxBytes, 10))
+		err := setPoolQuota(context, clusterInfo, pool.Name, "max_bytes", strconv.FormatUint(*pool.Quotas.MaxBytes, 10))
 		if err != nil {
-			return errors.Wrapf(err, "failed to set max_bytes quota for pool %q", poolName)
+			return errors.Wrapf(err, "failed to set max_bytes quota for pool %q", pool.Name)
 		}
 	}
 	// set max_objects quota
 	if pool.Quotas.MaxObjects != nil {
 		// set max_objects quota, 0 value disables quota
-		err := setPoolQuota(context, clusterInfo, poolName, "max_objects", strconv.FormatUint(*pool.Quotas.MaxObjects, 10))
+		err := setPoolQuota(context, clusterInfo, pool.Name, "max_objects", strconv.FormatUint(*pool.Quotas.MaxObjects, 10))
 		if err != nil {
-			return errors.Wrapf(err, "failed to set max_objects quota for pool %q", poolName)
+			return errors.Wrapf(err, "failed to set max_objects quota for pool %q", pool.Name)
 		}
 	}
 
@@ -350,30 +384,33 @@ func GetErasureCodeProfileForPool(baseName string) string {
 	return fmt.Sprintf("%s_ecprofile", baseName)
 }
 
-func CreateECPoolForApp(context *clusterd.Context, clusterInfo *ClusterInfo, poolName, ecProfileName string, pool cephv1.PoolSpec, pgCount, appName string, enableECOverwrite bool) error {
-	args := []string{"osd", "pool", "create", poolName, pgCount, "erasure", ecProfileName}
+func CreateECPoolForApp(context *clusterd.Context, clusterInfo *ClusterInfo, ecProfileName string, pool cephv1.NamedPoolSpec, pgCount, appName string, enableECOverwrite bool) error {
+	args := []string{"osd", "pool", "create", pool.Name, pgCount, "erasure", ecProfileName}
 	output, err := NewCephCommand(context, clusterInfo, args).Run()
 	if err != nil {
-		return errors.Wrapf(err, "failed to create EC pool %s. %s", poolName, string(output))
+		return errors.Wrapf(err, "failed to create EC pool %s. %s", pool.Name, string(output))
 	}
 
 	if enableECOverwrite {
-		if err = SetPoolProperty(context, clusterInfo, poolName, "allow_ec_overwrites", "true"); err != nil {
-			return errors.Wrapf(err, "failed to allow EC overwrite for pool %s", poolName)
+		if err = SetPoolProperty(context, clusterInfo, pool.Name, "allow_ec_overwrites", "true"); err != nil {
+			return errors.Wrapf(err, "failed to allow EC overwrite for pool %s", pool.Name)
 		}
 	}
 
-	if err = setCommonPoolProperties(context, clusterInfo, pool, poolName, appName); err != nil {
+	if err = setCommonPoolProperties(context, clusterInfo, pool, appName); err != nil {
 		return err
 	}
 
-	logger.Infof("creating EC pool %s succeeded", poolName)
+	logger.Infof("creating EC pool %s succeeded", pool.Name)
 	return nil
 }
 
-func CreateReplicatedPoolForApp(context *clusterd.Context, clusterInfo *ClusterInfo, clusterSpec *cephv1.ClusterSpec, poolName string, pool cephv1.PoolSpec, pgCount, appName string) error {
+func CreateReplicatedPoolForApp(context *clusterd.Context, clusterInfo *ClusterInfo, clusterSpec *cephv1.ClusterSpec, pool cephv1.NamedPoolSpec, pgCount, appName string) error {
+	// If it's a replicated pool, ensure the failure domain is desired
+	checkFailureDomain := false
+
 	// The crush rule name is the same as the pool unless we have a stretch cluster.
-	crushRuleName := poolName
+	crushRuleName := pool.Name
 	if clusterSpec.IsStretchCluster() {
 		// A stretch cluster enforces using the same crush rule for all pools.
 		// The stretch cluster rule is created initially by the operator when the stretch cluster is configured
@@ -381,43 +418,124 @@ func CreateReplicatedPoolForApp(context *clusterd.Context, clusterInfo *ClusterI
 		crushRuleName = defaultStretchCrushRuleName
 	} else if pool.IsHybridStoragePool() {
 		// Create hybrid crush rule
-		err := createHybridCrushRule(context, clusterInfo, clusterSpec, crushRuleName, pool)
+		err := createHybridCrushRule(context, clusterInfo, clusterSpec, crushRuleName, pool.PoolSpec)
 		if err != nil {
 			return errors.Wrapf(err, "failed to create hybrid crush rule %q", crushRuleName)
 		}
 	} else {
 		if pool.Replicated.ReplicasPerFailureDomain > 1 {
 			// Create a two-step CRUSH rule for pools other than stretch clusters
-			err := createStretchCrushRule(context, clusterInfo, clusterSpec, crushRuleName, pool)
+			err := createStretchCrushRule(context, clusterInfo, clusterSpec, crushRuleName, pool.PoolSpec)
 			if err != nil {
 				return errors.Wrapf(err, "failed to create two-step crush rule %q", crushRuleName)
 			}
 		} else {
 			// create a crush rule for a replicated pool, if a failure domain is specified
+			checkFailureDomain = true
 			if err := createReplicationCrushRule(context, clusterInfo, clusterSpec, crushRuleName, pool); err != nil {
 				return errors.Wrapf(err, "failed to create replicated crush rule %q", crushRuleName)
 			}
 		}
 	}
 
-	args := []string{"osd", "pool", "create", poolName, pgCount, "replicated", crushRuleName, "--size", strconv.FormatUint(uint64(pool.Replicated.Size), 10)}
+	args := []string{"osd", "pool", "create", pool.Name, pgCount, "replicated", crushRuleName, "--size", strconv.FormatUint(uint64(pool.Replicated.Size), 10)}
 	output, err := NewCephCommand(context, clusterInfo, args).Run()
 	if err != nil {
-		return errors.Wrapf(err, "failed to create replicated pool %s. %s", poolName, string(output))
+		return errors.Wrapf(err, "failed to create replicated pool %s. %s", pool.Name, string(output))
 	}
 
 	if !clusterSpec.IsStretchCluster() {
 		// the pool is type replicated, set the size for the pool now that it's been created
-		if err := SetPoolReplicatedSizeProperty(context, clusterInfo, poolName, strconv.FormatUint(uint64(pool.Replicated.Size), 10)); err != nil {
-			return errors.Wrapf(err, "failed to set size property to replicated pool %q to %d", poolName, pool.Replicated.Size)
+		// Only set the size if not 0, otherwise ceph will fail to set size to 0
+		if pool.Replicated.Size > 0 {
+			if err := SetPoolReplicatedSizeProperty(context, clusterInfo, pool.Name, strconv.FormatUint(uint64(pool.Replicated.Size), 10)); err != nil {
+				return errors.Wrapf(err, "failed to set size property to replicated pool %q to %d", pool.Name, pool.Replicated.Size)
+			}
 		}
 	}
 
-	if err = setCommonPoolProperties(context, clusterInfo, pool, poolName, appName); err != nil {
+	if err = setCommonPoolProperties(context, clusterInfo, pool, appName); err != nil {
 		return err
 	}
 
-	logger.Infof("creating replicated pool %s succeeded", poolName)
+	logger.Infof("creating replicated pool %s succeeded", pool.Name)
+
+	if checkFailureDomain {
+		if err = ensureFailureDomain(context, clusterInfo, clusterSpec, pool); err != nil {
+			return nil
+		}
+	}
+	return nil
+}
+
+func ensureFailureDomain(context *clusterd.Context, clusterInfo *ClusterInfo, clusterSpec *cephv1.ClusterSpec, pool cephv1.NamedPoolSpec) error {
+	if pool.FailureDomain == "" {
+		logger.Debugf("skipping check for failure domain on pool %q as it is not specified", pool.Name)
+		return nil
+	}
+
+	logger.Debugf("checking that pool %q has the failure domain %q", pool.Name, pool.FailureDomain)
+	details, err := GetPoolDetails(context, clusterInfo, pool.Name)
+	if err != nil {
+		return errors.Wrapf(err, "failed to get pool %q details", pool.Name)
+	}
+
+	// Find the failure domain for the current crush rule
+	rule, err := getCrushRule(context, clusterInfo, details.CrushRule)
+	if err != nil {
+		return errors.Wrapf(err, "failed to get crush rule %q", details.CrushRule)
+	}
+	currentFailureDomain := extractFailureDomain(rule)
+	if currentFailureDomain == pool.FailureDomain {
+		logger.Debugf("pool %q has the expected failure domain %q", pool.Name, pool.FailureDomain)
+		return nil
+	}
+	if currentFailureDomain == "" {
+		logger.Warningf("failure domain not found for crush rule %q, proceeding to create a new crush rule", details.CrushRule)
+	}
+
+	// Use a crush rule name that is unique to the desired failure domain
+	crushRuleName := fmt.Sprintf("%s_%s", pool.Name, pool.FailureDomain)
+	logger.Infof("updating pool %q failure domain from %q to %q with new crush rule %q", pool.Name, currentFailureDomain, pool.FailureDomain, crushRuleName)
+	logger.Infof("crush rule %q will no longer be used by pool %q", details.CrushRule, pool.Name)
+
+	// Create a new crush rule for the expected failure domain
+	if err := createReplicationCrushRule(context, clusterInfo, clusterSpec, crushRuleName, pool); err != nil {
+		return errors.Wrapf(err, "failed to create replicated crush rule %q", crushRuleName)
+	}
+
+	// Update the crush rule on the pool
+	if err := setCrushRule(context, clusterInfo, pool.Name, crushRuleName); err != nil {
+		return errors.Wrapf(err, "failed to set crush rule on pool %q", pool.Name)
+	}
+
+	logger.Infof("Successfully updated pool %q failure domain to %q", pool.Name, pool.FailureDomain)
+	return nil
+}
+
+func extractFailureDomain(rule ruleSpec) string {
+	// find the failure domain in the crush rule, which is the first step where the
+	// "type" property is set
+	for i, step := range rule.Steps {
+		if step.Type != "" {
+			return step.Type
+		}
+		// We expect the rule to be found by the second step, or else it is a more
+		// complex rule that would not be supported for updating the failure domain
+		if i == 1 {
+			break
+		}
+	}
+	return ""
+}
+
+func setCrushRule(context *clusterd.Context, clusterInfo *ClusterInfo, poolName, crushRule string) error {
+	args := []string{"osd", "pool", "set", poolName, "crush_rule", crushRule}
+
+	_, err := NewCephCommand(context, clusterInfo, args).Run()
+	if err != nil {
+		return errors.Wrapf(err, "failed to set crush rule %q", crushRule)
+	}
 	return nil
 }
 
@@ -548,7 +666,7 @@ func updateCrushMap(context *clusterd.Context, clusterInfo *ClusterInfo, ruleset
 	return nil
 }
 
-func createReplicationCrushRule(context *clusterd.Context, clusterInfo *ClusterInfo, clusterSpec *cephv1.ClusterSpec, ruleName string, pool cephv1.PoolSpec) error {
+func createReplicationCrushRule(context *clusterd.Context, clusterInfo *ClusterInfo, clusterSpec *cephv1.ClusterSpec, ruleName string, pool cephv1.NamedPoolSpec) error {
 	failureDomain := pool.FailureDomain
 	if failureDomain == "" {
 		failureDomain = cephv1.DefaultFailureDomain
