@@ -21,6 +21,7 @@ import (
 	"os"
 	"strings"
 
+	kp "github.com/IBM/keyprotect-go-client"
 	"github.com/coreos/pkg/capnslog"
 	"github.com/hashicorp/vault/api"
 	"github.com/libopenstorage/secrets"
@@ -47,14 +48,14 @@ type Config struct {
 	Provider    string
 	context     *clusterd.Context
 	clusterSpec *cephv1.ClusterSpec
-	clusterInfo *cephclient.ClusterInfo
+	ClusterInfo *cephclient.ClusterInfo
 }
 
 // NewConfig returns the selected KMS
 func NewConfig(context *clusterd.Context, clusterSpec *cephv1.ClusterSpec, clusterInfo *cephclient.ClusterInfo) *Config {
 	config := &Config{
 		context:     context,
-		clusterInfo: clusterInfo,
+		ClusterInfo: clusterInfo,
 		clusterSpec: clusterSpec,
 	}
 
@@ -64,6 +65,8 @@ func NewConfig(context *clusterd.Context, clusterSpec *cephv1.ClusterSpec, clust
 		config.Provider = secrets.TypeK8s
 	case secrets.TypeVault:
 		config.Provider = secrets.TypeVault
+	case TypeIBM:
+		config.Provider = TypeIBM
 	default:
 		logger.Errorf("unsupported kms type %q", Provider)
 	}
@@ -83,16 +86,34 @@ func (c *Config) PutSecret(secretName, secretValue string) error {
 	}
 	if c.IsVault() {
 		// Store the secret in Vault
-		v, err := InitVault(c.context, c.clusterInfo.Namespace, c.clusterSpec.Security.KeyManagementService.ConnectionDetails)
+		v, err := InitVault(c.context, c.ClusterInfo.Namespace, c.clusterSpec.Security.KeyManagementService.ConnectionDetails)
 		if err != nil {
 			return errors.Wrap(err, "failed to init vault kms")
 		}
-		k := buildKeyContext(c.clusterSpec.Security.KeyManagementService.ConnectionDetails)
+		k := buildVaultKeyContext(c.clusterSpec.Security.KeyManagementService.ConnectionDetails)
 		err = put(v, GenerateOSDEncryptionSecretName(secretName), secretValue, k)
 		if err != nil {
 			return errors.Wrap(err, "failed to put secret in vault")
 		}
 	}
+	if c.IsIBMKeyProtect() {
+		kpClient, err := InitKeyProtect(c.clusterSpec.Security.KeyManagementService.ConnectionDetails)
+		if err != nil {
+			return errors.Wrap(err, "failed to init ibm key protect")
+		}
+
+		// Create the key is not present
+		keyAlias := []string{secretName}
+		_, err = kpClient.CreateImportedKeyWithAliases(c.ClusterInfo.Context, secretName, nil, secretValue, "", "", true, keyAlias)
+		if err != nil {
+			if strings.Contains(err.Error(), "KEY_ALIAS_NOT_UNIQUE_ERR") {
+				logger.Debugf("key %q already exists. %v", secretName, err)
+				return nil
+			}
+
+			return errors.Wrap(err, "failed to put secret in ibm key protect")
+		}
+	}
 
 	return nil
 }
@@ -102,17 +123,28 @@ func (c *Config) GetSecret(secretName string) (string, error) {
 	var value string
 	if c.IsVault() {
 		// Store the secret in Vault
-		v, err := InitVault(c.context, c.clusterInfo.Namespace, c.clusterSpec.Security.KeyManagementService.ConnectionDetails)
+		v, err := InitVault(c.context, c.ClusterInfo.Namespace, c.clusterSpec.Security.KeyManagementService.ConnectionDetails)
 		if err != nil {
-			return "", errors.Wrap(err, "failed to get secret in vault")
+			return "", errors.Wrap(err, "failed to init vault")
 		}
 
-		k := buildKeyContext(c.clusterSpec.Security.KeyManagementService.ConnectionDetails)
+		k := buildVaultKeyContext(c.clusterSpec.Security.KeyManagementService.ConnectionDetails)
 		value, err = get(v, GenerateOSDEncryptionSecretName(secretName), k)
 		if err != nil {
-			return "", errors.Wrap(err, "failed to get secret in vault")
+			return "", errors.Wrap(err, "failed to get secret from vault")
 		}
 	}
+	if c.IsIBMKeyProtect() {
+		kpClient, err := InitKeyProtect(c.clusterSpec.Security.KeyManagementService.ConnectionDetails)
+		if err != nil {
+			return "", errors.Wrap(err, "failed to init ibm key protect")
+		}
+		keyObject, err := kpClient.GetKey(c.ClusterInfo.Context, secretName)
+		if err != nil {
+			return "", errors.Wrap(err, "failed to get secret from ibm key protect")
+		}
+		value = string(keyObject.Payload)
+	}
 
 	return value, nil
 }
@@ -121,21 +153,49 @@ func (c *Config) GetSecret(secretName string) (string, error) {
 func (c *Config) DeleteSecret(secretName string) error {
 	if c.IsVault() {
 		// Store the secret in Vault
-		v, err := InitVault(c.context, c.clusterInfo.Namespace, c.clusterSpec.Security.KeyManagementService.ConnectionDetails)
+		v, err := InitVault(c.context, c.ClusterInfo.Namespace, c.clusterSpec.Security.KeyManagementService.ConnectionDetails)
 		if err != nil {
 			return errors.Wrap(err, "failed to delete secret in vault")
 		}
 
-		k := buildKeyContext(c.clusterSpec.Security.KeyManagementService.ConnectionDetails)
+		k := buildVaultKeyContext(c.clusterSpec.Security.KeyManagementService.ConnectionDetails)
 
 		// Force removal of all the versions of the secret on K/V version 2
 		k[secrets.DestroySecret] = "true"
 
-		err = delete(v, GenerateOSDEncryptionSecretName(secretName), k)
+		err = deleteSecret(v, GenerateOSDEncryptionSecretName(secretName), k)
 		if err != nil {
 			return errors.Wrap(err, "failed to delete secret in vault")
 		}
 	}
+	if c.IsIBMKeyProtect() {
+		kpClient, err := InitKeyProtect(c.clusterSpec.Security.KeyManagementService.ConnectionDetails)
+		if err != nil {
+			return errors.Wrap(err, "failed to init ibm key protect")
+		}
+
+		// We use context.TODO() since the clusterInfo context has been cancelled by the CephCluster's
+		// deletion event
+		ctx := context.TODO()
+
+		// Fetch the key to get the ID
+		key, err := kpClient.GetKey(ctx, secretName)
+		if err != nil {
+			return errors.Wrap(err, "failed to get secret in ibm key protect")
+		}
+
+		// DeleteKey does not support deleting secret with the alias name so we must use the ID
+		// After you delete a key, the key transitions to the Destroyed state. Any data encrypted by
+		// keys in this state is no longer accessible. Metadata that is associated with the key,
+		// such as the key's deletion date, is kept in the Key Protect database. Destroyed keys can
+		// be recovered after up to 30 days or their expiration date, whichever is sooner. After 30
+		// days, keys can no longer be recovered, and become eligible to be purged after 90 days, a
+		// process that shreds the key material and makes its metadata inaccessible.
+		_, err = kpClient.DeleteKey(ctx, key.ID, kp.ReturnRepresentation, []kp.CallOpt{kp.ForceOpt{Force: true}}...)
+		if err != nil {
+			return errors.Wrap(err, "failed to delete secret in ibm key protect")
+		}
+	}
 
 	return nil
 }
@@ -149,11 +209,19 @@ func GetParam(kmsConfig map[string]string, param string) string {
 }
 
 // ValidateConnectionDetails validates mandatory KMS connection details
-func ValidateConnectionDetails(clusterdContext *clusterd.Context, securitySpec *cephv1.SecuritySpec, ns string) error {
-	ctx := context.TODO()
-	// A token must be specified
-	if !securitySpec.KeyManagementService.IsTokenAuthEnabled() {
-		return errors.New("failed to validate kms configuration (missing token in spec)")
+func ValidateConnectionDetails(ctx context.Context, clusterdContext *clusterd.Context, securitySpec *cephv1.SecuritySpec, ns string) error {
+	// Lookup mandatory connection details
+	for _, config := range kmsMandatoryConnectionDetails {
+		if GetParam(securitySpec.KeyManagementService.ConnectionDetails, config) == "" {
+			return errors.Errorf("failed to validate kms config %q. cannot be empty", config)
+		}
+	}
+
+	// A token must be specified if token-auth is used
+	if !securitySpec.KeyManagementService.IsK8sAuthEnabled() && securitySpec.KeyManagementService.TokenSecretName == "" {
+		if !securitySpec.KeyManagementService.IsTokenAuthEnabled() {
+			return errors.New("failed to validate kms configuration (missing token in spec)")
+		}
 	}
 
 	// KMS provider must be specified
@@ -166,32 +234,35 @@ func ValidateConnectionDetails(clusterdContext *clusterd.Context, securitySpec *
 			return errors.Wrapf(err, "failed to fetch kms token secret %q", securitySpec.KeyManagementService.TokenSecretName)
 		}
 
-		// Check for empty token
-		token, ok := kmsToken.Data[KMSTokenSecretNameKey]
-		if !ok || len(token) == 0 {
-			return errors.Errorf("failed to read k8s kms secret %q key %q (not found or empty)", KMSTokenSecretNameKey, securitySpec.KeyManagementService.TokenSecretName)
-		}
-
 		switch provider {
-		case "vault":
+		case secrets.TypeVault:
+			// Check for empty token
+			token, ok := kmsToken.Data[KMSTokenSecretNameKey]
+			if !ok || len(token) == 0 {
+				return errors.Errorf("failed to read k8s kms secret %q key %q (not found or empty)", KMSTokenSecretNameKey, securitySpec.KeyManagementService.TokenSecretName)
+			}
+
 			// Set the env variable
 			err = os.Setenv(api.EnvVaultToken, string(token))
 			if err != nil {
 				return errors.Wrap(err, "failed to set vault kms token to an env var")
 			}
+
+		case TypeIBM:
+			for _, config := range kmsIBMKeyProtectMandatoryTokenDetails {
+				v, ok := kmsToken.Data[config]
+				if !ok || len(v) == 0 {
+					return errors.Errorf("failed to read k8s kms secret %q key %q (not found or empty)", config, securitySpec.KeyManagementService.TokenSecretName)
+				}
+				// Append the token secret details to the connection details
+				securitySpec.KeyManagementService.ConnectionDetails[config] = strings.TrimSuffix(strings.TrimSpace(string(v)), "\n")
+			}
 		}
 	}
 
-	// Lookup mandatory connection details
-	for _, config := range kmsMandatoryConnectionDetails {
-		if GetParam(securitySpec.KeyManagementService.ConnectionDetails, config) == "" {
-			return errors.Errorf("failed to validate kms config %q. cannot be empty", config)
-		}
-	}
-
-	// Validate KMS provider connection details
+	// Validate KMS provider connection details for each provider
 	switch provider {
-	case "vault":
+	case secrets.TypeVault:
 		err := validateVaultConnectionDetails(clusterdContext, ns, securitySpec.KeyManagementService.ConnectionDetails)
 		if err != nil {
 			return errors.Wrap(err, "failed to validate vault connection details")
@@ -209,6 +280,14 @@ func ValidateConnectionDetails(clusterdContext *clusterd.Context, securitySpec *
 				securitySpec.KeyManagementService.ConnectionDetails[vault.VaultBackendKey] = backendVersion
 			}
 		}
+
+	case TypeIBM:
+		for _, config := range kmsIBMKeyProtectMandatoryConnectionDetails {
+			if GetParam(securitySpec.KeyManagementService.ConnectionDetails, config) == "" {
+				return errors.Errorf("failed to validate kms config %q. cannot be empty", config)
+			}
+		}
+
 	default:
 		return errors.Errorf("failed to validate kms provider connection details (provider %q not supported)", provider)
 	}
@@ -217,8 +296,7 @@ func ValidateConnectionDetails(clusterdContext *clusterd.Context, securitySpec *
 }
 
 // SetTokenToEnvVar sets a KMS token as an env variable
-func SetTokenToEnvVar(clusterdContext *clusterd.Context, tokenSecretName, provider, namespace string) error {
-	ctx := context.TODO()
+func SetTokenToEnvVar(ctx context.Context, clusterdContext *clusterd.Context, tokenSecretName, provider, namespace string) error {
 	// Get the secret containing the kms token
 	kmsToken, err := clusterdContext.Clientset.CoreV1().Secrets(namespace).Get(ctx, tokenSecretName, metav1.GetOptions{})
 	if err != nil {
