@@ -23,7 +23,6 @@ import (
 	"path/filepath"
 	"strconv"
 
-	"github.com/libopenstorage/secrets"
 	"github.com/pkg/errors"
 	cephv1 "github.com/rook/rook/pkg/apis/ceph.rook.io/v1"
 	kms "github.com/rook/rook/pkg/daemon/ceph/osd/kms"
@@ -193,85 +192,6 @@ if cryptsetup luksDump $BLOCK_PATH|grep -qEs "Version:.*2"; then
 else
 	echo "LUKS version is not 2 so not setting label and subsystem"
 fi
-`
-	// #nosec G101 no leak just variable names
-	getKEKFromVaultWithToken = `
-# DO NOT RUN WITH -x TO AVOID LEAKING VAULT_TOKEN
-set -e
-
-KEK_NAME=%s
-KEY_PATH=%s
-CURL_PAYLOAD=$(mktemp)
-ARGS=(--silent --show-error --request GET --header "X-Vault-Token: ${VAULT_TOKEN//[$'\t\r\n']}")
-PYTHON_DATA_PARSE="['data']"
-
-# If a vault namespace is set
-if [ -n "$VAULT_NAMESPACE" ]; then
-  ARGS+=(--header "X-Vault-Namespace: ${VAULT_NAMESPACE}")
-fi
-
-# If SSL is configured but self-signed CA is used
-if [ -n "$VAULT_SKIP_VERIFY" ] && [[ "$VAULT_SKIP_VERIFY" == "true" ]]; then
-  ARGS+=(--insecure)
-fi
-
-# TLS args
-if [ -n "$VAULT_CACERT" ]; then
-  if [ -z "$VAULT_CLIENT_CERT" ] && [ -z "$VAULT_CLIENT_KEY" ]; then
-    ARGS+=(--cacert "${VAULT_CACERT}")
-  else
-    ARGS+=(--capath $(dirname "${VAULT_CACERT}"))
-  fi
-fi
-if [ -n "$VAULT_CLIENT_CERT" ]; then
-  ARGS+=(--cert "${VAULT_CLIENT_CERT}")
-fi
-if [ -n "$VAULT_CLIENT_KEY" ]; then
-  ARGS+=(--key "${VAULT_CLIENT_KEY}")
-fi
-
-# For a request to any host/port, connect to VAULT_TLS_SERVER_NAME:requests original port instead
-# Used for SNI validation and correct certificate matching
-if [ -n "$VAULT_TLS_SERVER_NAME" ]; then
-  ARGS+=(--connect-to ::"${VAULT_TLS_SERVER_NAME}":)
-fi
-
-# trim VAULT_BACKEND_PATH for last character '/' to avoid a redirect response from the server
-VAULT_BACKEND_PATH="${VAULT_BACKEND_PATH%%/}"
-
-# Check KV engine version
-if [[ "$VAULT_BACKEND" == "v2" ]]; then
-  PYTHON_DATA_PARSE="['data']['data']"
-  VAULT_BACKEND_PATH="$VAULT_BACKEND_PATH/data"
-fi
-
-
-# Get the Key Encryption Key
-curl "${ARGS[@]}" "$VAULT_ADDR"/v1/"$VAULT_BACKEND_PATH"/"$KEK_NAME" > "$CURL_PAYLOAD"
-
-# Check for warnings in the payload
-if warning=$(python3 -c "import sys, json; print(json.load(sys.stdin)[\"warnings\"], end='')" 2> /dev/null < "$CURL_PAYLOAD"); then
-  if [[ "$warning" != None ]]; then
-    # We could get a warning but it is not necessary an issue, so if there is no key we exit
-    if ! python3 -c "import sys, json; print(json.load(sys.stdin)${PYTHON_DATA_PARSE}[\"$KEK_NAME\"], end='')" &> /dev/null < "$CURL_PAYLOAD"; then
-      echo "no encryption key $KEK_NAME present in vault"
-      echo "$warning"
-      exit 1
-    fi
-  fi
-fi
-
-# Check for errors in the payload
-if error=$(python3 -c "import sys, json; print(json.load(sys.stdin)[\"errors\"], end='')" 2> /dev/null < "$CURL_PAYLOAD"); then
-  echo "$error"
-  exit 1
-fi
-
-# Put the KEK in a file for cryptsetup to read
-python3 -c "import sys, json; print(json.load(sys.stdin)${PYTHON_DATA_PARSE}[\"$KEK_NAME\"], end='')" < "$CURL_PAYLOAD" > "$KEY_PATH"
-
-# purge payload file
-rm -f "$CURL_PAYLOAD"
 `
 
 	// If the disk identifier changes (different major and minor) we must force copy
@@ -347,7 +267,7 @@ func (c *Cluster) makeDeployment(osdProps osdProperties, osd OSDInfo, provisionC
 	volumes := controller.PodVolumes(provisionConfig.DataPathMap, dataDirHostPath, false)
 	failureDomainValue := osdProps.crushHostname
 	doConfigInit := true     // initialize ceph.conf in init container?
-	doBinaryCopyInit := true // copy tini and rook binaries in an init container?
+	doBinaryCopyInit := true // copy rook binary in an init container?
 
 	// This property is used for both PVC and non-PVC use case
 	if osd.CVMode == "" {
@@ -376,7 +296,7 @@ func (c *Cluster) makeDeployment(osdProps osdProperties, osd OSDInfo, provisionC
 			// Somehow when this happens and we try to update a deployment spec it fails with:
 			//  ValidationError(Pod.spec.volumes[7].projected): missing required field "sources"
 			if c.spec.Security.KeyManagementService.IsEnabled() && c.spec.Security.KeyManagementService.IsTLSEnabled() {
-				encryptedVol, _ := kms.VaultVolumeAndMount(c.spec.Security.KeyManagementService.ConnectionDetails)
+				encryptedVol, _ := kms.VaultVolumeAndMount(c.spec.Security.KeyManagementService.ConnectionDetails, "")
 				volumes = append(volumes, encryptedVol)
 			}
 		}
@@ -387,10 +307,7 @@ func (c *Cluster) makeDeployment(osdProps osdProperties, osd OSDInfo, provisionC
 	}
 
 	osdID := strconv.Itoa(osd.ID)
-	tiniEnvVar := v1.EnvVar{Name: "TINI_SUBREAPER", Value: ""}
-	envVars := append(c.getConfigEnvVars(osdProps, dataDir), []v1.EnvVar{
-		tiniEnvVar,
-	}...)
+	envVars := c.getConfigEnvVars(osdProps, dataDir)
 	envVars = append(envVars, k8sutil.ClusterDaemonEnvVars(c.spec.CephVersion.Image)...)
 	envVars = append(envVars, []v1.EnvVar{
 		{Name: "ROOK_OSD_UUID", Value: osd.UUID},
@@ -406,7 +323,6 @@ func (c *Cluster) makeDeployment(osdProps osdProperties, osd OSDInfo, provisionC
 		dataDeviceClassEnvVar(osd.DeviceClass),
 	}...)
 	configEnvVars := append(c.getConfigEnvVars(osdProps, dataDir), []v1.EnvVar{
-		tiniEnvVar,
 		{Name: "ROOK_OSD_ID", Value: osdID},
 		{Name: "ROOK_CEPH_VERSION", Value: c.clusterInfo.CephVersion.CephVersionFormatted()},
 		{Name: "ROOK_IS_DEVICE", Value: "true"},
@@ -418,9 +334,9 @@ func (c *Cluster) makeDeployment(osdProps osdProperties, osd OSDInfo, provisionC
 	// If the OSD was prepared with ceph-volume and running on PVC and using the LVM mode
 	if osdProps.onPVC() && osd.CVMode == "lvm" {
 		// if the osd was provisioned by ceph-volume, we need to launch it with rook as the parent process
-		command = []string{path.Join(rookBinariesMountPath, "tini")}
+		command = []string{path.Join(rookBinariesMountPath, "rook")}
 		args = []string{
-			"--", path.Join(rookBinariesMountPath, "rook"),
+			path.Join(rookBinariesMountPath, "rook"),
 			"ceph", "osd", "start",
 			"--",
 			"--foreground",
@@ -550,6 +466,7 @@ func (c *Cluster) makeDeployment(osdProps osdProperties, osd OSDInfo, provisionC
 				Image:           c.rookVersion,
 				VolumeMounts:    configVolumeMounts,
 				Env:             configEnvVars,
+				EnvFrom:         getEnvFromSources(),
 				SecurityContext: securityContext,
 			})
 	}
@@ -633,8 +550,10 @@ func (c *Cluster) makeDeployment(osdProps osdProperties, osd OSDInfo, provisionC
 					Image:           c.spec.CephVersion.Image,
 					VolumeMounts:    volumeMounts,
 					Env:             envVars,
+					EnvFrom:         getEnvFromSources(),
 					Resources:       osdProps.resources,
 					SecurityContext: securityContext,
+					StartupProbe:    controller.GenerateStartupProbeExecDaemon(opconfig.OsdType, osdID),
 					LivenessProbe:   controller.GenerateLivenessProbeExecDaemon(opconfig.OsdType, osdID),
 					WorkingDir:      opconfig.VarLogCephDir,
 				},
@@ -654,7 +573,7 @@ func (c *Cluster) makeDeployment(osdProps osdProperties, osd OSDInfo, provisionC
 		podTemplateSpec.Spec.Containers = append(podTemplateSpec.Spec.Containers, *controller.LogCollectorContainer(fmt.Sprintf("ceph-osd.%s", osdID), c.clusterInfo.Namespace, c.spec))
 	}
 
-	// If the liveness probe is enabled
+	podTemplateSpec.Spec.Containers[0] = opconfig.ConfigureStartupProbe(cephv1.KeyOSD, podTemplateSpec.Spec.Containers[0], c.spec.HealthCheck)
 	podTemplateSpec.Spec.Containers[0] = opconfig.ConfigureLivenessProbe(cephv1.KeyOSD, podTemplateSpec.Spec.Containers[0], c.spec.HealthCheck)
 
 	if c.spec.Network.IsHost() {
@@ -774,8 +693,8 @@ func applyTopologyAffinity(spec *v1.PodSpec, osd OSDInfo) error {
 	return nil
 }
 
-// To get rook inside the container, the config init container needs to copy "tini" and "rook" binaries into a volume.
-// Get the config flag so rook will copy the binaries and create the volume and mount that will be shared between
+// To get rook inside the container, the config init container needs to copy "rook" binary into a volume.
+// Get the config flag so rook will copy the binary and create the volume and mount that will be shared between
 // the init container and the daemon container
 func (c *Cluster) getCopyBinariesContainer() (v1.Volume, *v1.Container) {
 	volume := v1.Volume{Name: rookBinariesVolumeName, VolumeSource: v1.VolumeSource{EmptyDir: &v1.EmptyDirVolumeSource{}}}
@@ -848,14 +767,31 @@ func (c *Cluster) getActivateOSDInitContainer(configDir, namespace, osdID string
 		Name:            "activate",
 		Image:           c.spec.CephVersion.Image,
 		VolumeMounts:    volMounts,
-		SecurityContext: PrivilegedContext(),
+		SecurityContext: controller.PrivilegedContext(true),
 		Env:             envVars,
+		EnvFrom:         getEnvFromSources(),
 		Resources:       osdProps.resources,
 	}
 
 	return volume, container
 }
 
+// The blockdevmapper container copies the device node file, which is regarded as a device special file.
+// To be able to perform this action, the CAP_MKNOD capability is required.
+// Provide a securityContext which requests the MKNOD capability for the container to function properly.
+func getBlockDevMapperContext() *v1.SecurityContext {
+	privileged := controller.HostPathRequiresPrivileged()
+
+	return &v1.SecurityContext{
+		Capabilities: &v1.Capabilities{
+			Add: []v1.Capability{
+				"MKNOD",
+			},
+		},
+		Privileged: &privileged,
+	}
+}
+
 // Currently we can't mount a block mode pv directly to a privileged container
 // So we mount it to a non privileged init container and then copy it to a common directory mounted inside init container
 // and the privileged provision container.
@@ -875,7 +811,7 @@ func (c *Cluster) getPVCInitContainer(osdProps osdProperties) v1.Container {
 			},
 		},
 		VolumeMounts:    []v1.VolumeMount{getPvcOSDBridgeMount(osdProps.pvc.ClaimName)},
-		SecurityContext: controller.PodSecurityContext(),
+		SecurityContext: getBlockDevMapperContext(),
 		Resources:       osdProps.resources,
 	}
 }
@@ -907,7 +843,7 @@ func (c *Cluster) getPVCInitContainerActivate(mountPath string, osdProps osdProp
 			},
 		},
 		VolumeMounts:    []v1.VolumeMount{getPvcOSDBridgeMountActivate(mountPath, osdProps.pvc.ClaimName)},
-		SecurityContext: controller.PodSecurityContext(),
+		SecurityContext: getBlockDevMapperContext(),
 		Resources:       osdProps.resources,
 	}
 }
@@ -924,21 +860,28 @@ func (c *Cluster) generateEncryptionOpenBlockContainer(resources v1.ResourceRequ
 			fmt.Sprintf(openEncryptedBlock, c.clusterInfo.FSID, pvcName, encryptionKeyPath(), encryptionBlockDestinationCopy(mountPath, blockType), encryptionDMName(pvcName, cryptBlockType), encryptionDMPath(pvcName, cryptBlockType)),
 		},
 		VolumeMounts:    []v1.VolumeMount{getPvcOSDBridgeMountActivate(mountPath, volumeMountPVCName), getDeviceMapperMount()},
-		SecurityContext: PrivilegedContext(),
+		SecurityContext: controller.PrivilegedContext(true),
 		Resources:       resources,
 	}
 }
 
 func (c *Cluster) generateVaultGetKEK(osdProps osdProperties) v1.Container {
+	keyName := osdProps.pvc.ClaimName
+	keyPath := encryptionKeyPath()
+	envVars := c.getConfigEnvVars(osdProps, "")
+	envVars = append(envVars, kms.ConfigToEnvVar(c.spec)...)
+
 	return v1.Container{
-		Name:  blockEncryptionKMSGetKEKInitContainer,
-		Image: c.spec.CephVersion.Image,
-		Command: []string{
-			"/bin/bash",
-			"-c",
-			fmt.Sprintf(getKEKFromVaultWithToken, kms.GenerateOSDEncryptionSecretName(osdProps.pvc.ClaimName), encryptionKeyPath()),
+		Name:    blockEncryptionKMSGetKEKInitContainer,
+		Image:   c.rookVersion,
+		Command: []string{"rook"},
+		Args: []string{
+			"key-management",
+			"get",
+			keyName,
+			keyPath,
 		},
-		Env:       kms.VaultConfigToEnvVar(c.spec),
+		Env:       envVars,
 		Resources: osdProps.resources,
 	}
 }
@@ -948,26 +891,21 @@ func (c *Cluster) getPVCEncryptionOpenInitContainerActivate(mountPath string, os
 
 	// If a KMS is enabled we need to add an init container to fetch the KEK
 	if c.spec.Security.KeyManagementService.IsEnabled() {
-		kmsProvider := kms.GetParam(c.spec.Security.KeyManagementService.ConnectionDetails, kms.Provider)
-		// Get Vault KEK from KMS container
-		if kmsProvider == secrets.TypeVault {
-			if c.spec.Security.KeyManagementService.IsTokenAuthEnabled() {
-				getKEKFromKMSContainer := c.generateVaultGetKEK(osdProps)
+		getKEKFromKMSContainer := c.generateVaultGetKEK(osdProps)
 
-				// Volume mount to store the encrypted key
-				_, volMount := c.getEncryptionVolume(osdProps)
-				getKEKFromKMSContainer.VolumeMounts = append(getKEKFromKMSContainer.VolumeMounts, volMount)
+		// Volume mount to store the encrypted key
+		_, volMount := c.getEncryptionVolume(osdProps)
+		getKEKFromKMSContainer.VolumeMounts = append(getKEKFromKMSContainer.VolumeMounts, volMount)
 
-				// Now let's see if there is a TLS config we need to mount as well
-				if c.spec.Security.KeyManagementService.IsTLSEnabled() {
-					_, vaultVolMount := kms.VaultVolumeAndMount(c.spec.Security.KeyManagementService.ConnectionDetails)
-					getKEKFromKMSContainer.VolumeMounts = append(getKEKFromKMSContainer.VolumeMounts, vaultVolMount)
-				}
-
-				// Add the container to the list of containers
-				containers = append(containers, getKEKFromKMSContainer)
+		if c.spec.Security.KeyManagementService.IsVaultKMS() {
+			// Now let's see if there is a TLS config we need to mount as well
+			if c.spec.Security.KeyManagementService.IsTLSEnabled() {
+				_, vaultVolMount := kms.VaultVolumeAndMount(c.spec.Security.KeyManagementService.ConnectionDetails, "")
+				getKEKFromKMSContainer.VolumeMounts = append(getKEKFromKMSContainer.VolumeMounts, vaultVolMount)
 			}
 		}
+		// Add the container to the list of containers
+		containers = append(containers, getKEKFromKMSContainer)
 	}
 
 	// Main block container
@@ -1009,7 +947,7 @@ func (c *Cluster) generateEncryptionCopyBlockContainer(resources v1.ResourceRequ
 		// volumeMountPVCName is crucial, especially when the block we copy is the metadata block
 		// its value must be the name of the block PV so that all init containers use the same bridge (the emptyDir shared by all the init containers)
 		VolumeMounts:    []v1.VolumeMount{getPvcOSDBridgeMountActivate(mountPath, volumeMountPVCName), getDeviceMapperMount()},
-		SecurityContext: controller.PodSecurityContext(),
+		SecurityContext: getBlockDevMapperContext(),
 		Resources:       resources,
 	}
 }
@@ -1056,7 +994,7 @@ func (c *Cluster) getPVCMetadataInitContainer(mountPath string, osdProps osdProp
 				Name:      fmt.Sprintf("%s-bridge", osdProps.metadataPVC.ClaimName),
 			},
 		},
-		SecurityContext: controller.PodSecurityContext(),
+		SecurityContext: getBlockDevMapperContext(),
 		Resources:       osdProps.resources,
 	}
 }
@@ -1090,7 +1028,7 @@ func (c *Cluster) getPVCMetadataInitContainerActivate(mountPath string, osdProps
 		// We need to call getPvcOSDBridgeMountActivate() so that we can copy the metadata block into the "main" empty dir
 		// This empty dir is passed along every init container
 		VolumeMounts:    []v1.VolumeMount{getPvcOSDBridgeMountActivate(mountPath, osdProps.pvc.ClaimName)},
-		SecurityContext: controller.PodSecurityContext(),
+		SecurityContext: getBlockDevMapperContext(),
 		Resources:       osdProps.resources,
 	}
 }
@@ -1116,7 +1054,7 @@ func (c *Cluster) getPVCWalInitContainer(mountPath string, osdProps osdPropertie
 				Name:      fmt.Sprintf("%s-bridge", osdProps.walPVC.ClaimName),
 			},
 		},
-		SecurityContext: controller.PodSecurityContext(),
+		SecurityContext: getBlockDevMapperContext(),
 		Resources:       osdProps.resources,
 	}
 }
@@ -1150,7 +1088,7 @@ func (c *Cluster) getPVCWalInitContainerActivate(mountPath string, osdProps osdP
 		// We need to call getPvcOSDBridgeMountActivate() so that we can copy the wal block into the "main" empty dir
 		// This empty dir is passed along every init container
 		VolumeMounts:    []v1.VolumeMount{getPvcOSDBridgeMountActivate(mountPath, osdProps.pvc.ClaimName)},
-		SecurityContext: controller.PodSecurityContext(),
+		SecurityContext: getBlockDevMapperContext(),
 		Resources:       osdProps.resources,
 	}
 }
@@ -1173,7 +1111,7 @@ func (c *Cluster) getActivatePVCInitContainer(osdProps osdProperties, osdID stri
 			},
 		},
 		VolumeMounts:    []v1.VolumeMount{getPvcOSDBridgeMountActivate(osdDataPath, osdProps.pvc.ClaimName)},
-		SecurityContext: PrivilegedContext(),
+		SecurityContext: controller.PrivilegedContext(true),
 		Resources:       osdProps.resources,
 	}
 
@@ -1201,7 +1139,7 @@ func (c *Cluster) getExpandPVCInitContainer(osdProps osdProperties, osdID string
 		},
 		Args:            []string{"bluefs-bdev-expand", "--path", osdDataPath},
 		VolumeMounts:    []v1.VolumeMount{getPvcOSDBridgeMountActivate(osdDataPath, osdProps.pvc.ClaimName)},
-		SecurityContext: PrivilegedContext(),
+		SecurityContext: controller.PrivilegedContext(true),
 		Resources:       osdProps.resources,
 	}
 }
@@ -1228,7 +1166,7 @@ func (c *Cluster) getExpandEncryptedPVCInitContainer(mountPath string, osdProps
 		},
 		Args:            []string{"--verbose", "resize", encryptionDMName(osdProps.pvc.ClaimName, DmcryptBlockType)},
 		VolumeMounts:    volMount,
-		SecurityContext: PrivilegedContext(),
+		SecurityContext: controller.PrivilegedContext(true),
 		Resources:       osdProps.resources,
 	}
 }
@@ -1258,7 +1196,7 @@ func (c *Cluster) getEncryptedStatusPVCInitContainer(mountPath string, osdProps
 		},
 		Args:            []string{"--verbose", "status", encryptionDMName(osdProps.pvc.ClaimName, DmcryptBlockType)},
 		VolumeMounts:    []v1.VolumeMount{getPvcOSDBridgeMountActivate(mountPath, osdProps.pvc.ClaimName)},
-		SecurityContext: PrivilegedContext(),
+		SecurityContext: controller.PrivilegedContext(true),
 		Resources:       osdProps.resources,
 	}
 }
