@@ -32,6 +32,7 @@ import (
 	"github.com/rook/rook/pkg/client/clientset/versioned/scheme"
 	"github.com/rook/rook/pkg/clusterd"
 	"github.com/rook/rook/pkg/daemon/ceph/client"
+	cephver "github.com/rook/rook/pkg/operator/ceph/version"
 	"github.com/rook/rook/pkg/operator/k8sutil"
 	"github.com/rook/rook/pkg/operator/test"
 	"github.com/rook/rook/pkg/util/dependents"
@@ -141,7 +142,7 @@ const (
 	dummyVersionsRaw          = `
 	{
 		"mon": {
-			"ceph version 14.2.8 (3a54b2b6d167d4a2a19e003a705696d4fe619afc) nautilus (stable)": 3
+			"ceph version 15.2.11 (0000000000000000) octopus (stable)": 3
 		}
 	}`
 	userCreateJSON = `{
@@ -347,8 +348,9 @@ func TestCephObjectStoreController(t *testing.T) {
 			client:              cl,
 			scheme:              s,
 			context:             c,
-			objectStoreChannels: make(map[string]*objectStoreHealth),
-			recorder:            k8sutil.NewEventReporter(record.NewFakeRecorder(5)),
+			objectStoreContexts: make(map[string]*objectStoreHealth),
+			recorder:            record.NewFakeRecorder(5),
+			opManagerContext:    context.TODO(),
 		}
 
 		return r
@@ -363,6 +365,10 @@ func TestCephObjectStoreController(t *testing.T) {
 		},
 	}
 
+	currentAndDesiredCephVersion = func(ctx context.Context, rookImage string, namespace string, jobName string, ownerInfo *k8sutil.OwnerInfo, context *clusterd.Context, cephClusterSpec *cephv1.ClusterSpec, clusterInfo *client.ClusterInfo) (*cephver.CephVersion, *cephver.CephVersion, error) {
+		return &cephver.Pacific, &cephver.Pacific, nil
+	}
+
 	t.Run("error - no ceph cluster", func(t *testing.T) {
 		r := setupNewEnvironment()
 
@@ -513,6 +519,7 @@ func TestCephObjectStoreController(t *testing.T) {
 		assert.NotEmpty(t, objectStore.Status.Info["endpoint"], objectStore)
 		assert.Equal(t, "http://rook-ceph-rgw-my-store.rook-ceph.svc:80", objectStore.Status.Info["endpoint"], objectStore)
 		assert.True(t, calledCommitConfigChanges)
+		assert.Equal(t, 16, r.clusterInfo.CephVersion.Major)
 	})
 }
 
@@ -677,8 +684,9 @@ func TestCephObjectStoreControllerMultisite(t *testing.T) {
 		client:              cl,
 		scheme:              s,
 		context:             c,
-		objectStoreChannels: make(map[string]*objectStoreHealth),
-		recorder:            k8sutil.NewEventReporter(record.NewFakeRecorder(5)),
+		objectStoreContexts: make(map[string]*objectStoreHealth),
+		recorder:            record.NewFakeRecorder(5),
+		opManagerContext:    ctx,
 	}
 
 	_, err := r.context.Clientset.CoreV1().Secrets(namespace).Create(ctx, secret, metav1.CreateOptions{})
@@ -691,6 +699,10 @@ func TestCephObjectStoreControllerMultisite(t *testing.T) {
 		},
 	}
 
+	currentAndDesiredCephVersion = func(ctx context.Context, rookImage string, namespace string, jobName string, ownerInfo *k8sutil.OwnerInfo, context *clusterd.Context, cephClusterSpec *cephv1.ClusterSpec, clusterInfo *client.ClusterInfo) (*cephver.CephVersion, *cephver.CephVersion, error) {
+		return &cephver.Pacific, &cephver.Pacific, nil
+	}
+
 	t.Run("create an object store", func(t *testing.T) {
 		res, err := r.Reconcile(ctx, req)
 		assert.NoError(t, err)
