@@ -17,7 +17,6 @@ limitations under the License.
 package object
 
 import (
-	"context"
 	"fmt"
 	"path"
 	"reflect"
@@ -28,8 +27,10 @@ import (
 	"github.com/pkg/errors"
 	cephv1 "github.com/rook/rook/pkg/apis/ceph.rook.io/v1"
 	"github.com/rook/rook/pkg/daemon/ceph/osd/kms"
+	"github.com/rook/rook/pkg/operator/ceph/config"
 	cephconfig "github.com/rook/rook/pkg/operator/ceph/config"
 	"github.com/rook/rook/pkg/operator/ceph/controller"
+	cephver "github.com/rook/rook/pkg/operator/ceph/version"
 	"github.com/rook/rook/pkg/operator/k8sutil"
 	apps "k8s.io/api/apps/v1"
 	v1 "k8s.io/api/core/v1"
@@ -38,7 +39,7 @@ import (
 )
 
 const (
-	livenessProbePath = "/swift/healthcheck"
+	readinessProbePath = "/swift/healthcheck"
 	// #nosec G101 since this is not leaking any hardcoded details
 	setupVaultTokenFile = `
 set -e
@@ -46,11 +47,11 @@ set -e
 VAULT_TOKEN_OLD_PATH=%s
 VAULT_TOKEN_NEW_PATH=%s
 
-cp --verbose $VAULT_TOKEN_OLD_PATH $VAULT_TOKEN_NEW_PATH
+cp --recursive --verbose $VAULT_TOKEN_OLD_PATH/..data/. $VAULT_TOKEN_NEW_PATH
 
-chmod --verbose 400 $VAULT_TOKEN_NEW_PATH
-
-chown --verbose ceph:ceph $VAULT_TOKEN_NEW_PATH
+chmod --recursive --verbose 400 $VAULT_TOKEN_NEW_PATH/*
+chmod --verbose 700 $VAULT_TOKEN_NEW_PATH
+chown --recursive --verbose ceph:ceph $VAULT_TOKEN_NEW_PATH
 `
 )
 
@@ -112,7 +113,7 @@ func (c *clusterConfig) makeRGWPodSpec(rgwConfig *rgwConfig) (v1.PodTemplateSpec
 	if c.clusterSpec.LogCollector.Enabled {
 		shareProcessNamespace := true
 		podSpec.ShareProcessNamespace = &shareProcessNamespace
-		podSpec.Containers = append(podSpec.Containers, *controller.LogCollectorContainer(strings.TrimPrefix(generateCephXUser(fmt.Sprintf("ceph-client.%s", rgwConfig.ResourceName)), "client."), c.clusterInfo.Namespace, *c.clusterSpec))
+		podSpec.Containers = append(podSpec.Containers, *controller.LogCollectorContainer(getDaemonName(rgwConfig), c.clusterInfo.Namespace, *c.clusterSpec))
 	}
 
 	// Replace default unreachable node toleration
@@ -158,8 +159,16 @@ func (c *clusterConfig) makeRGWPodSpec(rgwConfig *rgwConfig) (v1.PodTemplateSpec
 	}
 	if kmsEnabled {
 		if c.store.Spec.Security.KeyManagementService.IsTokenAuthEnabled() {
-			podSpec.Volumes = append(podSpec.Volumes,
-				kms.VaultTokenFileVolume(c.store.Spec.Security.KeyManagementService.TokenSecretName))
+			vaultFileVol, _ := kms.VaultVolumeAndMount(c.store.Spec.Security.KeyManagementService.ConnectionDetails,
+				c.store.Spec.Security.KeyManagementService.TokenSecretName)
+			tmpvolume := v1.Volume{
+				Name: rgwVaultVolumeName,
+				VolumeSource: v1.VolumeSource{
+					EmptyDir: &v1.EmptyDirVolumeSource{},
+				},
+			}
+
+			podSpec.Volumes = append(podSpec.Volumes, vaultFileVol, tmpvolume)
 			podSpec.InitContainers = append(podSpec.InitContainers,
 				c.vaultTokenInitContainer(rgwConfig))
 		}
@@ -212,25 +221,26 @@ func (c *clusterConfig) createCaBundleUpdateInitContainer(rgwConfig *rgwConfig)
 }
 
 // The vault token is passed as Secret for rgw container. So it is mounted as read only.
-// RGW has restrictions over vault token file, it should owned by same user(ceph) which
+// RGW has restrictions over vault token file, it should owned by same user (ceph) which
 // rgw daemon runs and all other permission should be nil or zero. Here ownership can be
 // changed with help of FSGroup but in openshift environments for security reasons it has
-// predefined value, so it won't work there. Hence the token file is copied to containerDataDir
-// from mounted secret then ownership/permissions are changed accordingly with help of a
-// init container.
+// predefined value, so it won't work there. Hence the token file and certs (if present)
+// are copied to other volume from mounted secrets then ownership/permissions are changed
+// accordingly with help of an init container.
 func (c *clusterConfig) vaultTokenInitContainer(rgwConfig *rgwConfig) v1.Container {
-	_, volMount := kms.VaultVolumeAndMount(c.store.Spec.Security.KeyManagementService.ConnectionDetails)
+	_, srcVaultVolMount := kms.VaultVolumeAndMount(c.store.Spec.Security.KeyManagementService.ConnectionDetails, "")
+	tmpVaultMount := v1.VolumeMount{Name: rgwVaultVolumeName, MountPath: rgwVaultDirName}
 	return v1.Container{
 		Name: "vault-initcontainer-token-file-setup",
 		Command: []string{
 			"/bin/bash",
 			"-c",
 			fmt.Sprintf(setupVaultTokenFile,
-				path.Join(kms.EtcVaultDir, kms.VaultFileName), path.Join(c.DataPathMap.ContainerDataDir, kms.VaultFileName)),
+				kms.EtcVaultDir, rgwVaultDirName),
 		},
 		Image: c.clusterSpec.CephVersion.Image,
 		VolumeMounts: append(
-			controller.DaemonVolumeMounts(c.DataPathMap, rgwConfig.ResourceName), volMount),
+			controller.DaemonVolumeMounts(c.DataPathMap, rgwConfig.ResourceName), srcVaultVolMount, tmpVaultMount),
 		Resources:       c.store.Spec.Gateway.Resources,
 		SecurityContext: controller.PodSecurityContext(),
 	}
@@ -271,14 +281,19 @@ func (c *clusterConfig) makeDaemonContainer(rgwConfig *rgwConfig) v1.Container {
 		),
 		Env:             controller.DaemonEnvVars(c.clusterSpec.CephVersion.Image),
 		Resources:       c.store.Spec.Gateway.Resources,
-		StartupProbe:    c.generateStartupProbe(),
-		LivenessProbe:   c.generateLiveProbe(),
+		StartupProbe:    c.defaultStartupProbe(),
+		LivenessProbe:   c.defaultLivenessProbe(),
+		ReadinessProbe:  c.defaultReadinessProbe(),
 		SecurityContext: controller.PodSecurityContext(),
 		WorkingDir:      cephconfig.VarLogCephDir,
 	}
 
+	// If the startup probe is enabled
+	configureStartupProbe(&container, c.store.Spec.HealthCheck)
 	// If the liveness probe is enabled
 	configureLivenessProbe(&container, c.store.Spec.HealthCheck)
+	// If the readiness probe is enabled
+	configureReadinessProbe(&container, c.store.Spec.HealthCheck)
 	if c.store.Spec.IsTLSEnabled() {
 		// Add a volume mount for the ssl certificate
 		mount := v1.VolumeMount{Name: certVolumeName, MountPath: certDir, ReadOnly: true}
@@ -304,12 +319,31 @@ func (c *clusterConfig) makeDaemonContainer(rgwConfig *rgwConfig) v1.Container {
 			container.Args = append(container.Args,
 				cephconfig.NewFlag("rgw crypt vault auth", kms.KMSTokenSecretNameKey),
 				cephconfig.NewFlag("rgw crypt vault token file",
-					path.Join(c.DataPathMap.ContainerDataDir, kms.VaultFileName)),
+					path.Join(rgwVaultDirName, kms.VaultFileName)),
 				cephconfig.NewFlag("rgw crypt vault prefix", c.vaultPrefixRGW()),
 				cephconfig.NewFlag("rgw crypt vault secret engine",
 					c.store.Spec.Security.KeyManagementService.ConnectionDetails[kms.VaultSecretEngineKey]),
 			)
 		}
+		if c.store.Spec.Security.KeyManagementService.IsTLSEnabled() &&
+			c.clusterInfo.CephVersion.IsAtLeast(cephver.CephVersion{Major: 16, Minor: 2, Extra: 6}) {
+			container.Args = append(container.Args,
+				cephconfig.NewFlag("rgw crypt vault verify ssl", "true"))
+			if kms.GetParam(c.store.Spec.Security.KeyManagementService.ConnectionDetails, api.EnvVaultClientCert) != "" {
+				container.Args = append(container.Args,
+					cephconfig.NewFlag("rgw crypt vault ssl clientcert", path.Join(rgwVaultDirName, kms.VaultCertFileName)))
+			}
+			if kms.GetParam(c.store.Spec.Security.KeyManagementService.ConnectionDetails, api.EnvVaultClientKey) != "" {
+				container.Args = append(container.Args,
+					cephconfig.NewFlag("rgw crypt vault ssl clientkey", path.Join(rgwVaultDirName, kms.VaultKeyFileName)))
+			}
+			if kms.GetParam(c.store.Spec.Security.KeyManagementService.ConnectionDetails, api.EnvVaultCACert) != "" {
+				container.Args = append(container.Args,
+					cephconfig.NewFlag("rgw crypt vault ssl cacert", path.Join(rgwVaultDirName, kms.VaultCAFileName)))
+			}
+		}
+		vaultVolMount := v1.VolumeMount{Name: rgwVaultVolumeName, MountPath: rgwVaultDirName}
+		container.VolumeMounts = append(container.VolumeMounts, vaultVolMount)
 	}
 	return container
 }
@@ -322,7 +356,7 @@ func configureLivenessProbe(container *v1.Container, healthCheck cephv1.BucketHe
 			// If the spec value is empty, let's use a default
 			if probe != nil {
 				// Set the liveness probe on the container to overwrite the default probe created by Rook
-				container.LivenessProbe = cephconfig.GetLivenessProbeWithDefaults(probe, container.LivenessProbe)
+				container.LivenessProbe = cephconfig.GetProbeWithDefaults(probe, container.LivenessProbe)
 			}
 		} else {
 			container.LivenessProbe = nil
@@ -330,35 +364,69 @@ func configureLivenessProbe(container *v1.Container, healthCheck cephv1.BucketHe
 	}
 }
 
-func (c *clusterConfig) generateLiveProbe() *v1.Probe {
+// configureReadinessProbe returns the desired readiness probe for a given daemon
+func configureReadinessProbe(container *v1.Container, healthCheck cephv1.BucketHealthCheckSpec) {
+	if ok := healthCheck.ReadinessProbe; ok != nil {
+		if !healthCheck.ReadinessProbe.Disabled {
+			probe := healthCheck.ReadinessProbe.Probe
+			// If the spec value is empty, let's use a default
+			if probe != nil {
+				// Set the readiness probe on the container to overwrite the default probe created by Rook
+				container.ReadinessProbe = cephconfig.GetProbeWithDefaults(probe, container.ReadinessProbe)
+			}
+		} else {
+			container.ReadinessProbe = nil
+		}
+	}
+}
+
+func configureStartupProbe(container *v1.Container, healthCheck cephv1.BucketHealthCheckSpec) {
+	if ok := healthCheck.StartupProbe; ok != nil {
+		if !healthCheck.StartupProbe.Disabled {
+			probe := healthCheck.StartupProbe.Probe
+			// If the spec value is empty, let's use a default
+			if probe != nil {
+				// Set the startup probe on the container to overwrite the default probe created by Rook
+				container.StartupProbe = cephconfig.GetProbeWithDefaults(probe, container.StartupProbe)
+			}
+		} else {
+			container.StartupProbe = nil
+		}
+	}
+}
+
+func (c *clusterConfig) defaultLivenessProbe() *v1.Probe {
 	return &v1.Probe{
 		Handler: v1.Handler{
-			HTTPGet: &v1.HTTPGetAction{
-				Path:   livenessProbePath,
-				Port:   c.generateLiveProbePort(),
-				Scheme: c.generateLiveProbeScheme(),
+			TCPSocket: &v1.TCPSocketAction{
+				Port: c.generateProbePort(),
 			},
 		},
 		InitialDelaySeconds: 10,
 	}
 }
 
-func (c *clusterConfig) generateStartupProbe() *v1.Probe {
+func (c *clusterConfig) defaultReadinessProbe() *v1.Probe {
 	return &v1.Probe{
 		Handler: v1.Handler{
 			HTTPGet: &v1.HTTPGetAction{
-				Path:   livenessProbePath,
-				Port:   c.generateLiveProbePort(), // same as liveness but allows for longer RGW startup times
-				Scheme: c.generateLiveProbeScheme(),
+				Path:   readinessProbePath,
+				Port:   c.generateProbePort(),
+				Scheme: c.generateReadinessProbeScheme(),
 			},
 		},
 		InitialDelaySeconds: 10,
-		PeriodSeconds:       10,
-		FailureThreshold:    18,
 	}
 }
 
-func (c *clusterConfig) generateLiveProbeScheme() v1.URIScheme {
+func (c *clusterConfig) defaultStartupProbe() *v1.Probe {
+	probe := c.defaultLivenessProbe()
+	probe.PeriodSeconds = 10
+	probe.FailureThreshold = 18
+	return probe
+}
+
+func (c *clusterConfig) generateReadinessProbeScheme() v1.URIScheme {
 	// Default to HTTP
 	uriScheme := v1.URISchemeHTTP
 
@@ -371,7 +439,7 @@ func (c *clusterConfig) generateLiveProbeScheme() v1.URIScheme {
 	return uriScheme
 }
 
-func (c *clusterConfig) generateLiveProbePort() intstr.IntOrString {
+func (c *clusterConfig) generateProbePort() intstr.IntOrString {
 	// The port the liveness probe needs to probe
 	// Assume we run on SDN by default
 	port := intstr.FromInt(int(rgwPortInternalPort))
@@ -403,7 +471,7 @@ func (c *clusterConfig) generateService(cephObjectStore *cephv1.CephObjectStore)
 		svc.Spec.ClusterIP = v1.ClusterIPNone
 	}
 
-	destPort := c.generateLiveProbePort()
+	destPort := c.generateProbePort()
 
 	// When the cluster is external we must use the same one as the gateways are listening on
 	if cephObjectStore.Spec.IsExternal() {
@@ -453,7 +521,7 @@ func (c *clusterConfig) reconcileExternalEndpoint(cephObjectStore *cephv1.CephOb
 		return errors.Wrapf(err, "failed to set owner reference to ceph object store endpoint %q", endpoint.Name)
 	}
 
-	_, err = k8sutil.CreateOrUpdateEndpoint(c.context.Clientset, cephObjectStore.Namespace, endpoint)
+	_, err = k8sutil.CreateOrUpdateEndpoint(c.clusterInfo.Context, c.context.Clientset, cephObjectStore.Namespace, endpoint)
 	if err != nil {
 		return errors.Wrapf(err, "failed to create or update object store %q endpoint", cephObjectStore.Name)
 	}
@@ -469,7 +537,7 @@ func (c *clusterConfig) reconcileService(cephObjectStore *cephv1.CephObjectStore
 		return "", errors.Wrapf(err, "failed to set owner reference to ceph object store service %q", service.Name)
 	}
 
-	svc, err := k8sutil.CreateOrUpdateService(c.context.Clientset, cephObjectStore.Namespace, service)
+	svc, err := k8sutil.CreateOrUpdateService(c.clusterInfo.Context, c.context.Clientset, cephObjectStore.Namespace, service)
 	if err != nil {
 		return "", errors.Wrapf(err, "failed to create or update object store %q service", cephObjectStore.Name)
 	}
@@ -500,7 +568,7 @@ func (c *clusterConfig) vaultPrefixRGW() string {
 
 func (c *clusterConfig) CheckRGWKMS() (bool, error) {
 	if c.store.Spec.Security != nil && c.store.Spec.Security.KeyManagementService.IsEnabled() {
-		err := kms.ValidateConnectionDetails(c.context, c.store.Spec.Security, c.store.Namespace)
+		err := kms.ValidateConnectionDetails(c.clusterInfo.Context, c.context, c.store.Spec.Security, c.store.Namespace)
 		if err != nil {
 			return false, err
 		}
@@ -556,7 +624,7 @@ func addPortToEndpoint(endpoints *v1.Endpoints, name string, port int32) {
 }
 
 func getLabels(name, namespace string, includeNewLabels bool) map[string]string {
-	labels := controller.CephDaemonAppLabels(AppName, namespace, "rgw", name, includeNewLabels)
+	labels := controller.CephDaemonAppLabels(AppName, namespace, config.RgwType, name, name, "cephobjectstores.ceph.rook.io", includeNewLabels)
 	labels["rook_object_store"] = name
 	return labels
 }
@@ -622,9 +690,13 @@ func (c *clusterConfig) generateVolumeSourceWithCaBundleSecret() (*v1.SecretVolu
 }
 
 func (c *clusterConfig) rgwTLSSecretType(secretName string) (v1.SecretType, error) {
-	rgwTlsSecret, err := c.context.Clientset.CoreV1().Secrets(c.clusterInfo.Namespace).Get(context.TODO(), secretName, metav1.GetOptions{})
+	rgwTlsSecret, err := c.context.Clientset.CoreV1().Secrets(c.clusterInfo.Namespace).Get(c.clusterInfo.Context, secretName, metav1.GetOptions{})
 	if rgwTlsSecret != nil {
 		return rgwTlsSecret.Type, nil
 	}
 	return "", errors.Wrapf(err, "no Kubernetes secrets referring TLS certificates found")
 }
+
+func getDaemonName(rgwConfig *rgwConfig) string {
+	return fmt.Sprintf("ceph-%s", generateCephXUser(rgwConfig.ResourceName))
+}
