@@ -29,7 +29,6 @@ import (
 	"github.com/pkg/errors"
 	cephv1 "github.com/rook/rook/pkg/apis/ceph.rook.io/v1"
 	"github.com/rook/rook/pkg/clusterd"
-	"github.com/rook/rook/pkg/operator/ceph/cluster/mgr"
 	"github.com/rook/rook/pkg/operator/ceph/cluster/mon"
 	"github.com/rook/rook/pkg/operator/ceph/config"
 	opcontroller "github.com/rook/rook/pkg/operator/ceph/controller"
@@ -71,36 +70,34 @@ type ReconcileCephBlockPool struct {
 	scheme            *runtime.Scheme
 	context           *clusterd.Context
 	clusterInfo       *cephclient.ClusterInfo
-	blockPoolChannels map[string]*blockPoolHealth
+	blockPoolContexts map[string]*blockPoolHealth
+	opManagerContext  context.Context
 }
 
 type blockPoolHealth struct {
-	stopChan          chan struct{}
-	monitoringRunning bool
+	internalCtx    context.Context
+	internalCancel context.CancelFunc
+	started        bool
 }
 
 // Add creates a new CephBlockPool Controller and adds it to the Manager. The Manager will set fields on the Controller
 // and Start it when the Manager is Started.
-func Add(mgr manager.Manager, context *clusterd.Context) error {
-	return add(mgr, newReconciler(mgr, context))
+func Add(mgr manager.Manager, context *clusterd.Context, opManagerContext context.Context, opConfig opcontroller.OperatorConfig) error {
+	return add(opManagerContext, mgr, newReconciler(mgr, context, opManagerContext))
 }
 
 // newReconciler returns a new reconcile.Reconciler
-func newReconciler(mgr manager.Manager, context *clusterd.Context) reconcile.Reconciler {
-	// Add the cephv1 scheme to the manager scheme so that the controller knows about it
-	mgrScheme := mgr.GetScheme()
-	if err := cephv1.AddToScheme(mgr.GetScheme()); err != nil {
-		panic(err)
-	}
+func newReconciler(mgr manager.Manager, context *clusterd.Context, opManagerContext context.Context) reconcile.Reconciler {
 	return &ReconcileCephBlockPool{
 		client:            mgr.GetClient(),
-		scheme:            mgrScheme,
+		scheme:            mgr.GetScheme(),
 		context:           context,
-		blockPoolChannels: make(map[string]*blockPoolHealth),
+		blockPoolContexts: make(map[string]*blockPoolHealth),
+		opManagerContext:  opManagerContext,
 	}
 }
 
-func add(mgr manager.Manager, r reconcile.Reconciler) error {
+func add(opManagerContext context.Context, mgr manager.Manager, r reconcile.Reconciler) error {
 	// Create a new controller
 	c, err := controller.New(controllerName, mgr, controller.Options{Reconciler: r})
 	if err != nil {
@@ -116,7 +113,7 @@ func add(mgr manager.Manager, r reconcile.Reconciler) error {
 
 	// Build Handler function to return the list of ceph block pool
 	// This is used by the watchers below
-	handlerFunc, err := opcontroller.ObjectToCRMapper(mgr.GetClient(), &cephv1.CephBlockPoolList{}, mgr.GetScheme())
+	handlerFunc, err := opcontroller.ObjectToCRMapper(opManagerContext, mgr.GetClient(), &cephv1.CephBlockPoolList{}, mgr.GetScheme())
 	if err != nil {
 		return err
 	}
@@ -147,7 +144,7 @@ func (r *ReconcileCephBlockPool) Reconcile(context context.Context, request reco
 func (r *ReconcileCephBlockPool) reconcile(request reconcile.Request) (reconcile.Result, error) {
 	// Fetch the CephBlockPool instance
 	cephBlockPool := &cephv1.CephBlockPool{}
-	err := r.client.Get(context.TODO(), request.NamespacedName, cephBlockPool)
+	err := r.client.Get(r.opManagerContext, request.NamespacedName, cephBlockPool)
 	if err != nil {
 		if kerrors.IsNotFound(err) {
 			logger.Debug("CephBlockPool resource not found. Ignoring since object must be deleted.")
@@ -164,7 +161,7 @@ func (r *ReconcileCephBlockPool) reconcile(request reconcile.Request) (reconcile
 	}
 
 	// Set a finalizer so we can do cleanup before the object goes away
-	err = opcontroller.AddFinalizerIfNotPresent(r.client, cephBlockPool)
+	err = opcontroller.AddFinalizerIfNotPresent(r.opManagerContext, r.client, cephBlockPool)
 	if err != nil {
 		return opcontroller.ImmediateRetryResult, errors.Wrap(err, "failed to add finalizer")
 	}
@@ -175,7 +172,7 @@ func (r *ReconcileCephBlockPool) reconcile(request reconcile.Request) (reconcile
 	}
 
 	// Make sure a CephCluster is present otherwise do nothing
-	cephCluster, isReadyToReconcile, cephClusterExists, reconcileResponse := opcontroller.IsReadyToReconcile(r.client, request.NamespacedName, controllerName)
+	cephCluster, isReadyToReconcile, cephClusterExists, reconcileResponse := opcontroller.IsReadyToReconcile(r.opManagerContext, r.client, request.NamespacedName, controllerName)
 	if !isReadyToReconcile {
 		// This handles the case where the Ceph Cluster is gone and we want to delete that CR
 		// We skip the deletePool() function since everything is gone already
@@ -188,7 +185,7 @@ func (r *ReconcileCephBlockPool) reconcile(request reconcile.Request) (reconcile
 			r.cancelMirrorMonitoring(cephBlockPool)
 
 			// Remove finalizer
-			err = opcontroller.RemoveFinalizer(r.client, cephBlockPool)
+			err = opcontroller.RemoveFinalizer(r.opManagerContext, r.client, cephBlockPool)
 			if err != nil {
 				return opcontroller.ImmediateRetryResult, errors.Wrap(err, "failed to remove finalizer")
 			}
@@ -200,7 +197,7 @@ func (r *ReconcileCephBlockPool) reconcile(request reconcile.Request) (reconcile
 	}
 
 	// Populate clusterInfo during each reconcile
-	clusterInfo, _, _, err := mon.LoadClusterInfo(r.context, request.NamespacedName.Namespace)
+	clusterInfo, _, _, err := mon.LoadClusterInfo(r.context, r.opManagerContext, request.NamespacedName.Namespace)
 	if err != nil {
 		return opcontroller.ImmediateRetryResult, errors.Wrap(err, "failed to populate cluster info")
 	}
@@ -210,14 +207,19 @@ func (r *ReconcileCephBlockPool) reconcile(request reconcile.Request) (reconcile
 	// Initialize the channel for this pool
 	// This allows us to track multiple CephBlockPool in the same namespace
 	blockPoolChannelKey := blockPoolChannelKeyName(cephBlockPool)
-	_, poolChannelExists := r.blockPoolChannels[blockPoolChannelKey]
-	if !poolChannelExists {
-		r.blockPoolChannels[blockPoolChannelKey] = &blockPoolHealth{
-			stopChan:          make(chan struct{}),
-			monitoringRunning: false,
+	_, blockPoolContextsExists := r.blockPoolContexts[blockPoolChannelKey]
+	if !blockPoolContextsExists {
+		internalCtx, internalCancel := context.WithCancel(r.opManagerContext)
+		r.blockPoolContexts[blockPoolChannelKey] = &blockPoolHealth{
+			internalCtx:    internalCtx,
+			internalCancel: internalCancel,
 		}
 	}
 
+	// Set the name of the Ceph pool to be reconciled internally, which
+	// may be different from the pool CR name.
+	cephBlockPool.Spec.Name = getCephName(*cephBlockPool)
+
 	// DELETE: the CR was deleted
 	if !cephBlockPool.GetDeletionTimestamp().IsZero() {
 		// If the ceph block pool is still in the map, we must remove it during CR deletion
@@ -225,7 +227,8 @@ func (r *ReconcileCephBlockPool) reconcile(request reconcile.Request) (reconcile
 		r.cancelMirrorMonitoring(cephBlockPool)
 
 		logger.Infof("deleting pool %q", cephBlockPool.Name)
-		err := deletePool(r.context, clusterInfo, cephBlockPool)
+		poolSpec := cephBlockPool.Spec.ToNamedPoolSpec()
+		err := deletePool(r.context, clusterInfo, &poolSpec)
 		if err != nil {
 			return opcontroller.ImmediateRetryResult, errors.Wrapf(err, "failed to delete pool %q. ", cephBlockPool.Name)
 		}
@@ -236,7 +239,7 @@ func (r *ReconcileCephBlockPool) reconcile(request reconcile.Request) (reconcile
 		}
 
 		// Remove finalizer
-		err = opcontroller.RemoveFinalizer(r.client, cephBlockPool)
+		err = opcontroller.RemoveFinalizer(r.opManagerContext, r.client, cephBlockPool)
 		if err != nil {
 			return opcontroller.ImmediateRetryResult, errors.Wrap(err, "failed to remove finalizer")
 		}
@@ -246,7 +249,7 @@ func (r *ReconcileCephBlockPool) reconcile(request reconcile.Request) (reconcile
 	}
 
 	// validate the pool settings
-	if err := ValidatePool(r.context, clusterInfo, &cephCluster.Spec, cephBlockPool); err != nil {
+	if err := validatePool(r.context, clusterInfo, &cephCluster.Spec, cephBlockPool); err != nil {
 		if strings.Contains(err.Error(), opcontroller.UninitializedCephConfigError) {
 			logger.Info(opcontroller.OperatorNotInitializedMessage)
 			return opcontroller.WaitForRequeueIfOperatorNotInitialized, nil
@@ -261,18 +264,6 @@ func (r *ReconcileCephBlockPool) reconcile(request reconcile.Request) (reconcile
 	}
 	r.clusterInfo.CephVersion = *cephVersion
 
-	// If the CephCluster has enabled the "pg_autoscaler" module and is running Nautilus
-	// we force the pg_autoscale_mode to "on"
-	_, propertyExists := cephBlockPool.Spec.Parameters[cephclient.PgAutoscaleModeProperty]
-	if mgr.IsModuleInSpec(cephCluster.Spec.Mgr.Modules, mgr.PgautoscalerModuleName) &&
-		!cephVersion.IsAtLeastOctopus() &&
-		!propertyExists {
-		if len(cephBlockPool.Spec.Parameters) == 0 {
-			cephBlockPool.Spec.Parameters = make(map[string]string)
-		}
-		cephBlockPool.Spec.Parameters[cephclient.PgAutoscaleModeProperty] = cephclient.PgAutoscaleModeOn
-	}
-
 	// CREATE/UPDATE
 	reconcileResponse, err = r.reconcileCreatePool(clusterInfo, &cephCluster.Spec, cephBlockPool)
 	if err != nil {
@@ -289,7 +280,8 @@ func (r *ReconcileCephBlockPool) reconcile(request reconcile.Request) (reconcile
 		return reconcile.Result{}, errors.Wrap(err, "failed to enable/disable stats collection for pool(s)")
 	}
 
-	checker := newMirrorChecker(r.context, r.client, r.clusterInfo, request.NamespacedName, &cephBlockPool.Spec, cephBlockPool.Name)
+	poolSpec := cephBlockPool.Spec.ToNamedPoolSpec()
+	checker := newMirrorChecker(r.context, r.client, r.clusterInfo, request.NamespacedName, &poolSpec)
 	// ADD PEERS
 	logger.Debug("reconciling create rbd mirror peer configuration")
 	if cephBlockPool.Spec.Mirroring.Enabled {
@@ -305,11 +297,11 @@ func (r *ReconcileCephBlockPool) reconcile(request reconcile.Request) (reconcile
 		// Run the goroutine to update the mirroring status
 		if !cephBlockPool.Spec.StatusCheck.Mirror.Disabled {
 			// Start monitoring of the pool
-			if r.blockPoolChannels[blockPoolChannelKey].monitoringRunning {
+			if r.blockPoolContexts[blockPoolChannelKey].started {
 				logger.Debug("pool monitoring go routine already running!")
 			} else {
-				r.blockPoolChannels[blockPoolChannelKey].monitoringRunning = true
-				go checker.checkMirroring(r.blockPoolChannels[blockPoolChannelKey].stopChan)
+				go checker.checkMirroring(r.blockPoolContexts[blockPoolChannelKey].internalCtx)
+				r.blockPoolContexts[blockPoolChannelKey].started = true
 			}
 		}
 
@@ -321,7 +313,7 @@ func (r *ReconcileCephBlockPool) reconcile(request reconcile.Request) (reconcile
 		}
 
 		// ReconcilePoolIDMap updates the `rook-ceph-csi-mapping-config` with local and peer cluster pool ID map
-		err = peermap.ReconcilePoolIDMap(r.context, r.clusterInfo, cephBlockPool)
+		err = peermap.ReconcilePoolIDMap(r.opManagerContext, r.context, r.clusterInfo, cephBlockPool)
 		if err != nil {
 			return reconcileResponse, errors.Wrapf(err, "failed to update pool ID mapping config for the pool %q", cephBlockPool.Name)
 		}
@@ -335,7 +327,7 @@ func (r *ReconcileCephBlockPool) reconcile(request reconcile.Request) (reconcile
 		updateStatus(r.client, request.NamespacedName, cephv1.ConditionReady, nil)
 
 		// Stop monitoring the mirroring status of this pool
-		if poolChannelExists && r.blockPoolChannels[blockPoolChannelKey].monitoringRunning {
+		if blockPoolContextsExists && r.blockPoolContexts[blockPoolChannelKey].started {
 			r.cancelMirrorMonitoring(cephBlockPool)
 			// Reset the MirrorHealthCheckSpec
 			checker.updateStatusMirroring(nil, nil, nil, "")
@@ -348,7 +340,8 @@ func (r *ReconcileCephBlockPool) reconcile(request reconcile.Request) (reconcile
 }
 
 func (r *ReconcileCephBlockPool) reconcileCreatePool(clusterInfo *cephclient.ClusterInfo, cephCluster *cephv1.ClusterSpec, cephBlockPool *cephv1.CephBlockPool) (reconcile.Result, error) {
-	err := createPool(r.context, clusterInfo, cephCluster, cephBlockPool)
+	poolSpec := cephBlockPool.Spec.ToNamedPoolSpec()
+	err := createPool(r.context, clusterInfo, cephCluster, &poolSpec)
 	if err != nil {
 		return opcontroller.ImmediateRetryResult, errors.Wrapf(err, "failed to create pool %q.", cephBlockPool.GetName())
 	}
@@ -357,19 +350,48 @@ func (r *ReconcileCephBlockPool) reconcileCreatePool(clusterInfo *cephclient.Clu
 	return reconcile.Result{}, nil
 }
 
+func getCephName(pool cephv1.CephBlockPool) string {
+	// If the name is not overridden in the pool spec.name, set it to the name of the pool CR
+	if pool.Spec.Name == "" {
+		return pool.Name
+	}
+	if pool.Spec.Name == pool.Name {
+		// The same pool name is set, nothing is being overridden
+		return pool.Name
+	}
+	// The pool names that can be allowed to be overridden are enforced in the crd schema
+	return pool.Spec.Name
+}
+
 // Create the pool
-func createPool(context *clusterd.Context, clusterInfo *cephclient.ClusterInfo, clusterSpec *cephv1.ClusterSpec, p *cephv1.CephBlockPool) error {
+func createPool(context *clusterd.Context, clusterInfo *cephclient.ClusterInfo, clusterSpec *cephv1.ClusterSpec, p *cephv1.NamedPoolSpec) error {
+	// Set the application name to rbd by default, but override for special pools
+	appName := poolApplicationNameRBD
+	if p.Name == "device_health_metrics" {
+		appName = "mgr_devicehealth"
+	} else if p.Name == ".nfs" {
+		appName = "nfs"
+	}
+
 	// create the pool
-	logger.Infof("creating pool %q in namespace %q", p.Name, p.Namespace)
-	if err := cephclient.CreatePoolWithProfile(context, clusterInfo, clusterSpec, p.Name, p.Spec, poolApplicationNameRBD); err != nil {
+	logger.Infof("creating pool %q in namespace %q", p.Name, clusterInfo.Namespace)
+	if err := cephclient.CreatePoolWithProfile(context, clusterInfo, clusterSpec, *p, appName); err != nil {
 		return errors.Wrapf(err, "failed to create pool %q", p.Name)
 	}
 
+	logger.Infof("initializing pool %q", p.Name)
+	args := []string{"pool", "init", p.Name}
+	output, err := cephclient.NewRBDCommand(context, clusterInfo, args).Run()
+	if err != nil {
+		return errors.Wrapf(err, "failed to initialize pool %q. %s", p.Name, string(output))
+	}
+	logger.Infof("successfully initialized pool %q", p.Name)
+
 	return nil
 }
 
 // Delete the pool
-func deletePool(context *clusterd.Context, clusterInfo *cephclient.ClusterInfo, p *cephv1.CephBlockPool) error {
+func deletePool(context *clusterd.Context, clusterInfo *cephclient.ClusterInfo, p *cephv1.NamedPoolSpec) error {
 	pools, err := cephclient.ListPoolSummaries(context, clusterInfo)
 	if err != nil {
 		return errors.Wrap(err, "failed to list pools")
@@ -399,8 +421,8 @@ func configureRBDStats(clusterContext *clusterd.Context, clusterInfo *cephclient
 	}
 	for _, cephBlockPool := range cephBlockPoolList.Items {
 		if cephBlockPool.GetDeletionTimestamp() == nil && cephBlockPool.Spec.EnableRBDStats {
-			// list of CephBlockPool with enableRBDStats set to true and not marked for deletion
-			enableStatsForPools = append(enableStatsForPools, cephBlockPool.Name)
+			// add to list of CephBlockPool with enableRBDStats set to true and not marked for deletion
+			enableStatsForPools = append(enableStatsForPools, getCephName(cephBlockPool))
 		}
 	}
 	logger.Debugf("RBD per-image IO statistics will be collected for pools: %v", enableStatsForPools)
@@ -425,12 +447,12 @@ func blockPoolChannelKeyName(p *cephv1.CephBlockPool) string {
 func (r *ReconcileCephBlockPool) cancelMirrorMonitoring(cephBlockPool *cephv1.CephBlockPool) {
 	channelKey := blockPoolChannelKeyName(cephBlockPool)
 
-	_, poolContextExists := r.blockPoolChannels[channelKey]
+	_, poolContextExists := r.blockPoolContexts[channelKey]
 	if poolContextExists {
-		// Close the channel to stop the mirroring status
-		close(r.blockPoolChannels[channelKey].stopChan)
+		// Cancel the context to stop the go routine
+		r.blockPoolContexts[channelKey].internalCancel()
 
 		// Remove ceph block pool from the map
-		delete(r.blockPoolChannels, channelKey)
+		delete(r.blockPoolContexts, channelKey)
 	}
 }
