@@ -39,12 +39,16 @@ type CephManifests interface {
 	GetBlockSnapshotClass(snapshotClassName, reclaimPolicy string) string
 	GetFileStorageSnapshotClass(snapshotClassName, reclaimPolicy string) string
 	GetFilesystem(name string, activeCount int) string
-	GetNFS(name, pool string, daemonCount int) string
+	GetNFS(name string, daemonCount int) string
+	GetNFSPool() string
 	GetRBDMirror(name string, daemonCount int) string
 	GetObjectStore(name string, replicaCount, port int, tlsEnable bool) string
 	GetObjectStoreUser(name, displayName, store, usercaps, maxsize string, maxbuckets, maxobjects int) string
 	GetBucketStorageClass(storeName, storageClassName, reclaimPolicy, region string) string
 	GetOBC(obcName, storageClassName, bucketName string, maxObject string, createBucket bool) string
+	GetOBCNotification(obcName, storageClassName, bucketName string, notificationName string, createBucket bool) string
+	GetBucketNotification(notificationName string, topicName string) string
+	GetBucketTopic(topicName string, storeName string, httpEndpointService string) string
 	GetClient(name string, caps map[string]string) string
 }
 
@@ -56,10 +60,10 @@ type CephManifestsMaster struct {
 // NewCephManifests gets the manifest type depending on the Rook version desired
 func NewCephManifests(settings *TestCephSettings) CephManifests {
 	switch settings.RookVersion {
-	case VersionMaster:
+	case LocalBuildTag:
 		return &CephManifestsMaster{settings}
-	case Version1_6:
-		return &CephManifestsV1_6{settings}
+	case Version1_7:
+		return &CephManifestsPreviousVersion{settings, &CephManifestsMaster{settings}}
 	}
 	panic(fmt.Errorf("unrecognized ceph manifest version: %s", settings.RookVersion))
 }
@@ -69,10 +73,7 @@ func (m *CephManifestsMaster) Settings() *TestCephSettings {
 }
 
 func (m *CephManifestsMaster) GetCRDs(k8shelper *utils.K8sHelper) string {
-	if k8shelper.VersionAtLeast("v1.16.0") {
-		return m.settings.readManifest("crds.yaml")
-	}
-	return m.settings.readManifest("pre-k8s-1.16/crds.yaml")
+	return m.settings.readManifest("crds.yaml")
 }
 
 func (m *CephManifestsMaster) GetOperator() string {
@@ -105,7 +106,7 @@ func (m *CephManifestsMaster) GetToolbox() string {
 //**********************************************************************************
 //**********************************************************************************
 // After a release, copy the methods below this separator into the versioned file
-// such as ceph_manifests_v1.6.go. Methods above this separator do not need to be
+// such as ceph_manifests_previous.go. Methods above this separator do not need to be
 // copied since the versioned implementation will load them directly from github.
 //**********************************************************************************
 //**********************************************************************************
@@ -275,7 +276,8 @@ spec:
     size: ` + replicaSize + `
     targetSizeRatio: .5
     requireSafeReplicaSize: false
-  compressionMode: aggressive
+  parameters:
+    compression_mode: aggressive
   mirroring:
     enabled: true
     mode: image
@@ -290,8 +292,7 @@ spec:
 
 func (m *CephManifestsMaster) GetBlockStorageClass(poolName, storageClassName, reclaimPolicy string) string {
 	// Create a CSI driver storage class
-	if m.settings.UseCSI {
-		return `
+	return `
 apiVersion: storage.k8s.io/v1
 kind: StorageClass
 metadata:
@@ -308,18 +309,6 @@ parameters:
   imageFeatures: layering
   csi.storage.k8s.io/fstype: ext4
 `
-	}
-	// Create a FLEX driver storage class
-	return `apiVersion: storage.k8s.io/v1
-kind: StorageClass
-metadata:
-   name: ` + storageClassName + `
-provisioner: ceph.rook.io/block
-allowVolumeExpansion: true
-reclaimPolicy: ` + reclaimPolicy + `
-parameters:
-    blockPool: ` + poolName + `
-    clusterNamespace: ` + m.settings.Namespace
 }
 
 func (m *CephManifestsMaster) GetFileStorageClass(fsName, storageClassName string) string {
@@ -366,7 +355,7 @@ spec:
 }
 
 // GetFilesystem returns the manifest to create a Rook Ceph NFS resource with the given config.
-func (m *CephManifestsMaster) GetNFS(name, pool string, count int) string {
+func (m *CephManifestsMaster) GetNFS(name string, count int) string {
 	return `apiVersion: ceph.rook.io/v1
 kind: CephNFS
 metadata:
@@ -374,12 +363,26 @@ metadata:
   namespace: ` + m.settings.Namespace + `
 spec:
   rados:
-    pool: ` + pool + `
+    pool: .nfs
     namespace: nfs-ns
   server:
     active: ` + strconv.Itoa(count)
 }
 
+// GetFilesystem returns the manifest to create a Rook Ceph NFS resource with the given config.
+func (m *CephManifestsMaster) GetNFSPool() string {
+	return `apiVersion: ceph.rook.io/v1
+kind: CephBlockPool
+metadata:
+  name: dot-nfs
+  namespace: ` + m.settings.Namespace + `
+spec:
+  name: .nfs
+  replicated:
+    size: 1
+    requireSafeReplicaSize: false`
+}
+
 func (m *CephManifestsMaster) GetObjectStore(name string, replicaCount, port int, tlsEnable bool) string {
 	if tlsEnable {
 		return `apiVersion: ceph.rook.io/v1
@@ -481,6 +484,47 @@ spec:
     maxObjects: "` + maxObject + `"`
 }
 
+//GetOBCNotification returns the manifest to create object bucket claim
+func (m *CephManifestsMaster) GetOBCNotification(claimName string, storageClassName string, objectBucketName string, notificationName string, varBucketName bool) string {
+	bucketParameter := "generateBucketName"
+	if varBucketName {
+		bucketParameter = "bucketName"
+	}
+	return `apiVersion: objectbucket.io/v1alpha1
+kind: ObjectBucketClaim
+metadata:
+  name: ` + claimName + `
+  labels:
+    bucket-notification-` + notificationName + `: ` + notificationName + `
+spec:
+  ` + bucketParameter + `: ` + objectBucketName + `
+  storageClassName: ` + storageClassName
+}
+
+//GetBucketNotification returns the manifest to create ceph bucket notification
+func (m *CephManifestsMaster) GetBucketNotification(notificationName string, topicName string) string {
+	return `apiVersion: ceph.rook.io/v1
+kind: CephBucketNotification
+metadata:
+  name: ` + notificationName + `
+spec:
+  topic: ` + topicName
+}
+
+//GetBucketTopic returns the manifest to create ceph bucket topic
+func (m *CephManifestsMaster) GetBucketTopic(topicName string, storeName string, httpEndpointService string) string {
+	return `apiVersion: ceph.rook.io/v1
+kind: CephBucketTopic
+metadata:
+  name: ` + topicName + `
+spec:
+  endpoint:
+    http:
+      uri: http://` + httpEndpointService + `
+  objectStoreName: ` + storeName + `
+  objectStoreNamespace: ` + m.settings.Namespace
+}
+
 func (m *CephManifestsMaster) GetClient(claimName string, caps map[string]string) string {
 	clientCaps := []string{}
 	for name, cap := range caps {
