@@ -17,6 +17,7 @@ limitations under the License.
 package integration
 
 import (
+	"context"
 	"fmt"
 	"strings"
 	"testing"
@@ -35,6 +36,10 @@ import (
 const (
 	rbdPodName        = "test-pod-upgrade"
 	operatorContainer = "rook-ceph-operator"
+	poolName          = "upgradepool"
+	storageClassName  = "block-upgrade"
+	blockName         = "block-claim-upgrade"
+	bucketPrefix      = "generate-me" // use generated bucket name for this test
 )
 
 // ************************************************
@@ -51,11 +56,6 @@ func TestCephUpgradeSuite(t *testing.T) {
 		t.Skip()
 	}
 
-	// Skip the suite if CSI is not supported
-	kh, err := utils.CreateK8sHelper(func() *testing.T { return t })
-	require.NoError(t, err)
-	checkSkipCSITest(t, kh)
-
 	s := new(UpgradeSuite)
 	defer func(s *UpgradeSuite) {
 		HandlePanics(recover(), s.TearDownSuite, s.T)
@@ -82,13 +82,12 @@ func (s *UpgradeSuite) SetupSuite() {
 		UseHelm:           false,
 		UsePVC:            false,
 		Mons:              1,
-		UseCSI:            true,
 		SkipOSDCreation:   false,
-		RookVersion:       installer.Version1_6,
-		CephVersion:       installer.NautilusPartitionVersion,
+		RookVersion:       installer.Version1_7,
+		CephVersion:       installer.OctopusVersion,
 	}
 
-	s.installer, s.k8sh = StartTestCluster(s.T, s.settings, upgradeMinimalTestVersion)
+	s.installer, s.k8sh = StartTestCluster(s.T, s.settings)
 	s.helper = clients.CreateTestClient(s.k8sh, s.installer.Manifests)
 }
 
@@ -96,100 +95,40 @@ func (s *UpgradeSuite) TearDownSuite() {
 	s.installer.UninstallRook()
 }
 
-func (s *UpgradeSuite) TestUpgradeToMaster() {
-	//
-	// Create block, object, and file storage before the upgrade
-	//
-	poolName := "upgradepool"
-	storageClassName := "block-upgrade"
-	blockName := "block-claim-upgrade"
-	logger.Infof("Initializing block before the upgrade")
-	clusterInfo := client.AdminClusterInfo(s.namespace)
-	setupBlockLite(s.helper, s.k8sh, s.Suite, clusterInfo, poolName, storageClassName, blockName, rbdPodName)
+func (s *UpgradeSuite) TestUpgradeRookToMaster() {
+	message := "my simple message"
+	objectStoreName := "upgraded-object"
+	objectUserID := "upgraded-user"
+	preFilename := "pre-upgrade-file"
+	numOSDs, filesystemName, rbdFilesToRead, cephfsFilesToRead := s.deployClusterforUpgrade(objectStoreName, objectUserID, message, preFilename)
+	s.settings.CephVersion = installer.OctopusVersion
 
-	createPodWithBlock(s.helper, s.k8sh, s.Suite, s.namespace, storageClassName, rbdPodName, blockName)
-
-	// FIX: We should require block images to be removed. See tracking issue:
-	// <INSERT ISSUE>
+	clusterInfo := client.AdminTestClusterInfo(s.namespace)
 	requireBlockImagesRemoved := false
-	defer blockTestDataCleanUp(s.helper, s.k8sh, s.Suite, clusterInfo, poolName, storageClassName, blockName, rbdPodName, requireBlockImagesRemoved)
-
-	// Create the filesystem
-	logger.Infof("Initializing file before the upgrade")
-	filesystemName := "upgrade-test-fs"
-	activeCount := 1
-	createFilesystem(s.helper, s.k8sh, s.Suite, s.settings, filesystemName, activeCount)
-
-	// Start the file test client
-	fsStorageClass := "file-upgrade"
-	assert.NoError(s.T(), s.helper.FSClient.CreateStorageClass(filesystemName, s.settings.OperatorNamespace, s.namespace, fsStorageClass))
-	createFilesystemConsumerPod(s.helper, s.k8sh, s.Suite, s.settings, filesystemName, fsStorageClass)
 	defer func() {
+		blockTestDataCleanUp(s.helper, s.k8sh, s.Suite, clusterInfo, poolName, storageClassName, blockName, rbdPodName, requireBlockImagesRemoved)
 		cleanupFilesystemConsumer(s.helper, s.k8sh, s.Suite, s.namespace, filePodName)
 		cleanupFilesystem(s.helper, s.k8sh, s.Suite, s.namespace, filesystemName)
-	}()
-
-	logger.Infof("Initializing object before the upgrade")
-	objectStoreName := "upgraded-object"
-	runObjectE2ETestLite(s.helper, s.k8sh, s.Suite, s.settings, objectStoreName, 1, false)
-
-	logger.Infof("Initializing object user before the upgrade")
-	objectUserID := "upgraded-user"
-	createCephObjectUser(s.Suite, s.helper, s.k8sh, s.namespace, objectStoreName, objectUserID, false, false)
-
-	logger.Info("Initializing object bucket claim before the upgrade")
-	bucketStorageClassName := "rook-smoke-delete-bucket"
-	bucketPrefix := "generate-me" // use generated bucket name for this test
-	cobErr := s.helper.BucketClient.CreateBucketStorageClass(s.namespace, objectStoreName, bucketStorageClassName, "Delete", region)
-	require.Nil(s.T(), cobErr)
-	cobcErr := s.helper.BucketClient.CreateObc(obcName, bucketStorageClassName, bucketPrefix, maxObject, false)
-	require.Nil(s.T(), cobcErr)
-	defer func() {
 		_ = s.helper.ObjectUserClient.Delete(s.namespace, objectUserID)
 		_ = s.helper.BucketClient.DeleteObc(obcName, bucketStorageClassName, bucketPrefix, maxObject, false)
 		_ = s.helper.BucketClient.DeleteBucketStorageClass(s.namespace, objectStoreName, bucketStorageClassName, "Delete", region)
 		objectStoreCleanUp(s.Suite, s.helper, s.k8sh, s.settings.Namespace, objectStoreName)
 	}()
 
-	created := utils.Retry(12, 2*time.Second, "OBC is created", func() bool {
-		// do not check if bound here b/c this fails in Rook v1.4
-		return s.helper.BucketClient.CheckOBC(obcName, "created")
-	})
-	require.True(s.T(), created)
-
-	// verify that we're actually running the right pre-upgrade image
-	s.verifyOperatorImage(installer.Version1_6)
-
-	message := "my simple message"
-	preFilename := "pre-upgrade-file"
-	assert.NoError(s.T(), s.k8sh.WriteToPod("", rbdPodName, preFilename, message))
-	assert.NoError(s.T(), s.k8sh.ReadFromPod("", rbdPodName, preFilename, message))
-
-	// we will keep appending to this to continue verifying old files through the upgrades
-	rbdFilesToRead := []string{preFilename}
-	cephfsFilesToRead := []string{}
-
-	// Get some info about the currently deployed OSDs to determine later if they are all updated
-	osdDepList, err := k8sutil.GetDeployments(s.k8sh.Clientset, s.namespace, "app=rook-ceph-osd")
-	require.NoError(s.T(), err)
-	osdDeps := osdDepList.Items
-	numOSDs := len(osdDeps) // there should be this many upgraded OSDs
-	require.NotEqual(s.T(), 0, numOSDs)
-
 	//
-	// Upgrade Rook from v1.6 to master
+	// Upgrade Rook from v1.7 to master
 	//
-	logger.Infof("*** UPGRADING ROOK FROM %s to master ***", installer.Version1_6)
+	logger.Infof("*** UPGRADING ROOK FROM %s to master ***", installer.Version1_7)
 	s.gatherLogs(s.settings.OperatorNamespace, "_before_master_upgrade")
 	s.upgradeToMaster()
 
-	s.verifyOperatorImage(installer.VersionMaster)
+	s.verifyOperatorImage(installer.LocalBuildTag)
 	s.verifyRookUpgrade(numOSDs)
-	err = s.installer.WaitForToolbox(s.namespace)
+	err := s.installer.WaitForToolbox(s.namespace)
 	assert.NoError(s.T(), err)
 
-	logger.Infof("Done with automatic upgrade from %s to master", installer.Version1_6)
-	newFile := "post-upgrade-1_6-to-master-file"
+	logger.Infof("Done with automatic upgrade from %s to master", installer.Version1_7)
+	newFile := "post-upgrade-1_7-to-master-file"
 	s.verifyFilesAfterUpgrade(filesystemName, newFile, message, rbdFilesToRead, cephfsFilesToRead)
 	rbdFilesToRead = append(rbdFilesToRead, newFile)
 	cephfsFilesToRead = append(cephfsFilesToRead, newFile)
@@ -200,20 +139,7 @@ func (s *UpgradeSuite) TestUpgradeToMaster() {
 	// do not need retry b/c the OBC controller runs parallel to Rook-Ceph orchestration
 	assert.True(s.T(), s.helper.BucketClient.CheckOBC(obcName, "bound"))
 
-	logger.Infof("Verified upgrade from %s to master", installer.Version1_6)
-
-	//
-	// Upgrade from nautilus to octopus
-	//
-	logger.Infof("*** UPGRADING CEPH FROM Nautilus TO Octopus ***")
-	s.gatherLogs(s.settings.OperatorNamespace, "_before_octopus_upgrade")
-	s.upgradeCephVersion(installer.OctopusVersion.Image, numOSDs)
-	// Verify reading and writing to the test clients
-	newFile = "post-octopus-upgrade-file"
-	s.verifyFilesAfterUpgrade(filesystemName, newFile, message, rbdFilesToRead, cephfsFilesToRead)
-	logger.Infof("Verified upgrade from nautilus to octopus")
-
-	checkCephObjectUser(s.Suite, s.helper, s.k8sh, s.namespace, objectStoreName, objectUserID, true, false)
+	logger.Infof("Verified upgrade from %s to master", installer.Version1_7)
 
 	//
 	// Upgrade from octopus to pacific
@@ -229,6 +155,133 @@ func (s *UpgradeSuite) TestUpgradeToMaster() {
 	checkCephObjectUser(s.Suite, s.helper, s.k8sh, s.namespace, objectStoreName, objectUserID, true, false)
 }
 
+func (s *UpgradeSuite) TestUpgradeCephToOctopusDevel() {
+	message := "my simple message"
+	objectStoreName := "upgraded-object"
+	objectUserID := "upgraded-user"
+	preFilename := "pre-upgrade-file"
+	s.settings.CephVersion = installer.OctopusVersion
+	numOSDs, filesystemName, rbdFilesToRead, cephfsFilesToRead := s.deployClusterforUpgrade(objectStoreName, objectUserID, message, preFilename)
+	clusterInfo := client.AdminTestClusterInfo(s.namespace)
+	requireBlockImagesRemoved := false
+	defer func() {
+		blockTestDataCleanUp(s.helper, s.k8sh, s.Suite, clusterInfo, poolName, storageClassName, blockName, rbdPodName, requireBlockImagesRemoved)
+		cleanupFilesystemConsumer(s.helper, s.k8sh, s.Suite, s.namespace, filePodName)
+		cleanupFilesystem(s.helper, s.k8sh, s.Suite, s.namespace, filesystemName)
+		_ = s.helper.ObjectUserClient.Delete(s.namespace, objectUserID)
+		_ = s.helper.BucketClient.DeleteObc(obcName, bucketStorageClassName, bucketPrefix, maxObject, false)
+		_ = s.helper.BucketClient.DeleteBucketStorageClass(s.namespace, objectStoreName, bucketStorageClassName, "Delete", region)
+		objectStoreCleanUp(s.Suite, s.helper, s.k8sh, s.settings.Namespace, objectStoreName)
+	}()
+
+	//
+	// Upgrade from octopus to octopus
+	//
+	logger.Infof("*** UPGRADING CEPH FROM OCTOPUS STABLE TO OCTOPUS DEVEL ***")
+	s.gatherLogs(s.settings.OperatorNamespace, "_before_pacific_upgrade")
+	s.upgradeCephVersion(installer.OctopusDevelVersion.Image, numOSDs)
+	// Verify reading and writing to the test clients
+	newFile := "post-octopus-upgrade-file"
+	s.verifyFilesAfterUpgrade(filesystemName, newFile, message, rbdFilesToRead, cephfsFilesToRead)
+	logger.Infof("Verified upgrade from octopus stable to octopus devel")
+
+	checkCephObjectUser(s.Suite, s.helper, s.k8sh, s.namespace, objectStoreName, objectUserID, true, false)
+}
+
+func (s *UpgradeSuite) TestUpgradeCephToPacificDevel() {
+	message := "my simple message"
+	objectStoreName := "upgraded-object"
+	objectUserID := "upgraded-user"
+	preFilename := "pre-upgrade-file"
+	s.settings.CephVersion = installer.PacificVersion
+	numOSDs, filesystemName, rbdFilesToRead, cephfsFilesToRead := s.deployClusterforUpgrade(objectStoreName, objectUserID, message, preFilename)
+	clusterInfo := client.AdminTestClusterInfo(s.namespace)
+	requireBlockImagesRemoved := false
+	defer func() {
+		blockTestDataCleanUp(s.helper, s.k8sh, s.Suite, clusterInfo, poolName, storageClassName, blockName, rbdPodName, requireBlockImagesRemoved)
+		cleanupFilesystemConsumer(s.helper, s.k8sh, s.Suite, s.namespace, filePodName)
+		cleanupFilesystem(s.helper, s.k8sh, s.Suite, s.namespace, filesystemName)
+		_ = s.helper.ObjectUserClient.Delete(s.namespace, objectUserID)
+		_ = s.helper.BucketClient.DeleteObc(obcName, bucketStorageClassName, bucketPrefix, maxObject, false)
+		_ = s.helper.BucketClient.DeleteBucketStorageClass(s.namespace, objectStoreName, bucketStorageClassName, "Delete", region)
+		objectStoreCleanUp(s.Suite, s.helper, s.k8sh, s.settings.Namespace, objectStoreName)
+	}()
+
+	//
+	// Upgrade from octopus to pacific
+	//
+	logger.Infof("*** UPGRADING CEPH FROM PACIFIC STABLE TO PACIFIC DEVEL ***")
+	s.gatherLogs(s.settings.OperatorNamespace, "_before_pacific_upgrade")
+	s.upgradeCephVersion(installer.PacificDevelVersion.Image, numOSDs)
+	// Verify reading and writing to the test clients
+	newFile := "post-pacific-upgrade-file"
+	s.verifyFilesAfterUpgrade(filesystemName, newFile, message, rbdFilesToRead, cephfsFilesToRead)
+	logger.Infof("Verified upgrade from pacific stable to pacific devel")
+
+	checkCephObjectUser(s.Suite, s.helper, s.k8sh, s.namespace, objectStoreName, objectUserID, true, false)
+}
+
+func (s *UpgradeSuite) deployClusterforUpgrade(objectStoreName, objectUserID, message, preFilename string) (int, string, []string, []string) {
+	//
+	// Create block, object, and file storage before the upgrade
+	//
+	logger.Infof("Initializing block before the upgrade")
+	clusterInfo := client.AdminTestClusterInfo(s.namespace)
+	setupBlockLite(s.helper, s.k8sh, s.Suite, clusterInfo, poolName, storageClassName, blockName, rbdPodName)
+
+	createPodWithBlock(s.helper, s.k8sh, s.Suite, s.namespace, storageClassName, rbdPodName, blockName)
+
+	// Create the filesystem
+	logger.Infof("Initializing file before the upgrade")
+	filesystemName := "upgrade-test-fs"
+	activeCount := 1
+	createFilesystem(s.helper, s.k8sh, s.Suite, s.settings, filesystemName, activeCount)
+
+	// Start the file test client
+	fsStorageClass := "file-upgrade"
+	assert.NoError(s.T(), s.helper.FSClient.CreateStorageClass(filesystemName, s.settings.OperatorNamespace, s.namespace, fsStorageClass))
+	createFilesystemConsumerPod(s.helper, s.k8sh, s.Suite, s.settings, filesystemName, fsStorageClass)
+
+	logger.Infof("Initializing object before the upgrade")
+	deleteStore := false
+	tls := false
+	runObjectE2ETestLite(s.T(), s.helper, s.k8sh, s.settings.Namespace, objectStoreName, 1, deleteStore, tls)
+
+	logger.Infof("Initializing object user before the upgrade")
+	createCephObjectUser(s.Suite, s.helper, s.k8sh, s.namespace, objectStoreName, objectUserID, false, false)
+
+	logger.Info("Initializing object bucket claim before the upgrade")
+	cobErr := s.helper.BucketClient.CreateBucketStorageClass(s.namespace, objectStoreName, bucketStorageClassName, "Delete", region)
+	require.Nil(s.T(), cobErr)
+	cobcErr := s.helper.BucketClient.CreateObc(obcName, bucketStorageClassName, bucketPrefix, maxObject, false)
+	require.Nil(s.T(), cobcErr)
+
+	created := utils.Retry(12, 2*time.Second, "OBC is created", func() bool {
+		// do not check if bound here b/c this fails in Rook v1.4
+		return s.helper.BucketClient.CheckOBC(obcName, "created")
+	})
+	require.True(s.T(), created)
+
+	// verify that we're actually running the right pre-upgrade image
+	s.verifyOperatorImage(installer.Version1_7)
+
+	assert.NoError(s.T(), s.k8sh.WriteToPod("", rbdPodName, preFilename, message))
+	assert.NoError(s.T(), s.k8sh.ReadFromPod("", rbdPodName, preFilename, message))
+
+	// we will keep appending to this to continue verifying old files through the upgrades
+	rbdFilesToRead := []string{preFilename}
+	cephfsFilesToRead := []string{}
+
+	// Get some info about the currently deployed OSDs to determine later if they are all updated
+	osdDepList, err := k8sutil.GetDeployments(context.TODO(), s.k8sh.Clientset, s.namespace, "app=rook-ceph-osd")
+	require.NoError(s.T(), err)
+	osdDeps := osdDepList.Items
+	numOSDs := len(osdDeps) // there should be this many upgraded OSDs
+	require.NotEqual(s.T(), 0, numOSDs)
+
+	return numOSDs, filesystemName, rbdFilesToRead, cephfsFilesToRead
+}
+
 func (s *UpgradeSuite) gatherLogs(systemNamespace, testSuffix string) {
 	// Gather logs before Ceph upgrade to help with debugging
 	if installer.TestLogCollectionLevel() == "all" {
@@ -239,7 +292,7 @@ func (s *UpgradeSuite) gatherLogs(systemNamespace, testSuffix string) {
 }
 
 func (s *UpgradeSuite) upgradeCephVersion(newCephImage string, numOSDs int) {
-	osdDepList, err := k8sutil.GetDeployments(s.k8sh.Clientset, s.namespace, "app=rook-ceph-osd")
+	osdDepList, err := k8sutil.GetDeployments(context.TODO(), s.k8sh.Clientset, s.namespace, "app=rook-ceph-osd")
 	require.NoError(s.T(), err)
 	oldCephVersion := osdDepList.Items[0].Labels["ceph-version"] // upgraded OSDs should not have this version label
 
@@ -254,24 +307,24 @@ func (s *UpgradeSuite) verifyOperatorImage(expectedImage string) {
 	systemNamespace := installer.SystemNamespace(s.namespace)
 
 	// verify that the operator spec is updated
-	version, err := k8sutil.GetDeploymentImage(s.k8sh.Clientset, systemNamespace, operatorContainer, operatorContainer)
+	version, err := k8sutil.GetDeploymentImage(context.TODO(), s.k8sh.Clientset, systemNamespace, operatorContainer, operatorContainer)
 	assert.NoError(s.T(), err)
 	assert.Equal(s.T(), "rook/ceph:"+expectedImage, version)
 }
 
 func (s *UpgradeSuite) verifyRookUpgrade(numOSDs int) {
 	// Get some info about the currently deployed mons to determine later if they are all updated
-	monDepList, err := k8sutil.GetDeployments(s.k8sh.Clientset, s.namespace, "app=rook-ceph-mon")
+	monDepList, err := k8sutil.GetDeployments(context.TODO(), s.k8sh.Clientset, s.namespace, "app=rook-ceph-mon")
 	require.NoError(s.T(), err)
 	require.Equal(s.T(), s.settings.Mons, len(monDepList.Items), monDepList.Items)
 
 	// Get some info about the currently deployed mgr to determine later if it is updated
-	mgrDepList, err := k8sutil.GetDeployments(s.k8sh.Clientset, s.namespace, "app=rook-ceph-mgr")
+	mgrDepList, err := k8sutil.GetDeployments(context.TODO(), s.k8sh.Clientset, s.namespace, "app=rook-ceph-mgr")
 	require.NoError(s.T(), err)
 	require.Equal(s.T(), 1, len(mgrDepList.Items))
 
 	// Get some info about the currently deployed OSDs to determine later if they are all updated
-	osdDepList, err := k8sutil.GetDeployments(s.k8sh.Clientset, s.namespace, "app=rook-ceph-osd")
+	osdDepList, err := k8sutil.GetDeployments(context.TODO(), s.k8sh.Clientset, s.namespace, "app=rook-ceph-osd")
 	require.NoError(s.T(), err)
 	require.NotZero(s.T(), len(osdDepList.Items))
 	require.Equal(s.T(), numOSDs, len(osdDepList.Items), osdDepList.Items)
@@ -340,7 +393,7 @@ func (s *UpgradeSuite) verifyFilesAfterUpgrade(fsName, newFileToWrite, messageFo
 
 	if fsName != "" {
 		// wait for filesystem to be active
-		clusterInfo := client.AdminClusterInfo(s.namespace)
+		clusterInfo := client.AdminTestClusterInfo(s.namespace)
 		err := waitForFilesystemActive(s.k8sh, clusterInfo, fsName)
 		require.NoError(s.T(), err)
 
@@ -359,15 +412,14 @@ func (s *UpgradeSuite) verifyFilesAfterUpgrade(fsName, newFileToWrite, messageFo
 // verify the upgrade but merely starts the upgrade process.
 func (s *UpgradeSuite) upgradeToMaster() {
 	// Apply the CRDs for the latest master
-	s.settings.RookVersion = installer.VersionMaster
+	s.settings.RookVersion = installer.LocalBuildTag
 	m := installer.NewCephManifests(s.settings)
 	require.NoError(s.T(), s.k8sh.ResourceOperation("apply", m.GetCRDs(s.k8sh)))
 
 	require.NoError(s.T(), s.k8sh.ResourceOperation("apply", m.GetCommon()))
 
 	require.NoError(s.T(),
-		s.k8sh.SetDeploymentVersion(s.settings.OperatorNamespace, operatorContainer, operatorContainer, installer.VersionMaster))
+		s.k8sh.SetDeploymentVersion(s.settings.OperatorNamespace, operatorContainer, operatorContainer, installer.LocalBuildTag))
 
-	require.NoError(s.T(),
-		s.k8sh.SetDeploymentVersion(s.settings.Namespace, "rook-ceph-tools", "rook-ceph-tools", installer.VersionMaster))
+	require.NoError(s.T(), s.k8sh.ResourceOperation("apply", m.GetToolbox()))
 }
